/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.condition.*;

class asdk.AbstractCollection implements Collection
{
	private var comparator : IComparator;
	
	// abstract
	function addItem( item ) : Boolean
	{
		throwError();
		return false;
	}
	
	// abstract
	public function iterator() : Iterator
	{
        trace( "in AC iterator" );
		throwError();
		return null;
	}

	// abstract
	function getSize() : Number
	{
		throwError();
		return -1;
	}
	
	function New() : Collection
	{
		throwError();
		return null;
	}
	
	function throwError()
	{
		throw new Error( "AbstractCollection: this method is abtract and not implemented" );
	}
	
	function setComparator( comparator : IComparator )
	{
		this.comparator = comparator;
	}
	
	function getComparator() : IComparator
	{
		if( this.comparator == undefined )
		  this.comparator = new ObjectComparator();
		  
		return this.comparator;
	}
	
	public function addAll( object ) : Boolean
	{
		var i : Iterator = IteratorFactory.createIterator( object );
		
		if( i.hasNext() )
		{
			while( i.hasNext() )
		  		addItem( i.getNext() );
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function clear() : Void
	{
		var i : Iterator = iterator();
		
		while( i.hasNext() )
		{
			i.getNext();
			i.remove();
		}
	}
	
	function containsItem( item ) : Boolean
	{
		var i : Iterator = iterator();

		while( i.hasNext() )
		  if( getComparator().equal( i.getNext(), item ) )
		    return true;
		
		return false;
	}
	
	function containsAllItems( items : Collection ) : Boolean
	{
		var i : Iterator = items.iterator();
		
		while( i.hasNext() )
		  if( !containsItem( i.getNext() ) )
		    return false;
		
		return true;
	}	
	
	function isEmpty() : Boolean
	{
		return getSize() == 0;
	}
	
	
	function removeItem( item ) : Boolean
	{
		var i : Iterator = iterator();
		
		while( i.hasNext() )
		{
		  if( getComparator().equal( i.getNext(), item ) )
		  {
			  i.remove();
			  return true;
		  }
		}
		
		return false;
	}
	
	function removeAllItems( items : Collection ) : Boolean
	{
		var collectionChanged : Boolean = false;
		
		if( items.getSize() > getSize() )
		{
			var i : Iterator = iterator();
			
			while( i.hasNext() )
			  if( items.containsItem( i.getNext() ) )
			  {
			    i.remove();
				collectionChanged = true;
			  }
		}
		else
		{
			var i : Iterator = items.iterator();
			
			while( i.hasNext() ){
				collectionChanged |= removeItem( i.getNext() );
			}
		}
		
		
		return collectionChanged;		  
	}
	
	function retainAllItems( items : Collection ) : Boolean
	{
		var i : Iterator = this.iterator();
		var collectionChanged : Boolean = false;
		
		while( i.hasNext() )
		{
			var item = i.getNext();
			
			if( !items.containsItem( item ) )
			   if( removeItem( item ) )
			     collectionChanged = true;
		}
		
		return collectionChanged;
	}
	
	function toArray() : Array
	{
		var newArray : Array = new Array();
		var i : Iterator = iterator();
		
		while( i.hasNext() )
		  newArray.push( i.getNext() );
		  
		return newArray;
	}
	
	public function toString():String
	{
		var debugString : String = "[";
		var iteratorObj : Iterator = iterator();
		
		while( iteratorObj.hasNext() )
		{
		  debugString += iteratorObj.getNext();
		  
		  if( iteratorObj.hasNext() )
		    debugString += ",";
		}
		
		return debugString + "]";
	}	
}