/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.iterator.*;

class asdk.LinkedList extends AbstractCollection
{
	var head : LinkedListItem;
	var tail : LinkedListItem;
	
	function LinkedList( object )
	{
		if( object == undefined )
			return;
			
		addAll( object );
	}
	
	function New() : Collection
	{
		return new LinkedList();
	}
	
	public function addItem( object )
	{
		var listItem : LinkedListItem = new LinkedListItem( object );
	
		if( this.head == null )
		{
			this.head = listItem;
		}
		else
		{
			this.tail.next = listItem;
			listItem.prev = this.tail;
		}
		
		this.tail = listItem;	
		return listItem;
	}
	
	public function iterator() : Iterator
	{
		return new LinkedListIterator( this, false );
	}
	
	function dataIterator() : Iterator
	{
		return new LinkedListIterator( this, true );
	}
	
	function reverseIterator() : Iterator
	{
		return new ReversedLinkedListIterator( this, false );
	}
	
	function reverseDataIterator( returnData : Boolean )
	{
		return new ReversedLinkedListIterator( this, true );
	}
	
	function removeItem( item ) : Boolean
	{
		var i : Iterator;
		
		if( item instanceof LinkedListItem )
			i = iterator();
		else
			i = dataIterator();
		
		while( i.hasNext() )
		{
		  if( getComparator().equal( i.getNext(), item ) )
		  {
			  i.remove();
			  return true;
		  }
		}
		
		return false;
	}

	function getSize() : Number
	{
		var itemCount : Number = 0;
		var listItem : LinkedListItem = this.head;
		
		for( ; listItem != undefined; itemCount++ )
			listItem = listItem.next;
		
		return itemCount;
	}	
	
	public function insertItemAt( index : Number, object ) : LinkedListItem
	{
		if( index == 0 )
		{
		  return addFirst( object );
		}
		
		var listItem : LinkedListItem = new LinkedListItem( object );
		var itemAtIndex = getItemAt( index );
		
		if( itemAtIndex == null )
		  itemAtIndex = tail;
		
		listItem.prev = itemAtIndex.prev;
		listItem.next = itemAtIndex;
		itemAtIndex.prev = listItem;
		listItem.prev.next = listItem;
		return listItem;
	}
    
    public static function insertBefore( node : LinkedListItem, object )
    {
        var listItem : LinkedListItem;
        
        if( !(object instanceof LinkedListItem) )
            listItem = new LinkedListItem( object );
        else
            listItem = object;        
        
        listItem.next = node;
        listItem.prev = node.prev;
        
        node.prev.next = listItem;
        node.prev = listItem;        
    }
    
    public static function replaceNodeWith( node : LinkedListItem, object )
    {
        var listItem : LinkedListItem;
        
        if( !(object instanceof LinkedListItem) )
            listItem = new LinkedListItem( object );
        else
            listItem = object;        
        
        listItem.prev = node.prev;
        listItem.next = node.next;
        
        node.prev.next = listItem;
        node.next.prev = listItem;
    }
	
	public function addFirst( object ) : LinkedListItem
	{
		var listItem : LinkedListItem = new LinkedListItem( object );
		
		if( head != null )
		{
			listItem.next = this.head;
			this.head.prev = listItem;
		}
		else
		{
			this.tail = listItem;
		}
		
		this.head = listItem;
		return this.head;
	}
	
	public function addLast( object ) : LinkedListItem
	{
		var listItem : LinkedListItem = new LinkedListItem( object );
		
		if( this.tail != null )
		{
			listItem.prev = this.tail;
			this.tail.next = listItem;
		}
		else
		{
			this.head = listItem;
		}
		
		this.tail = listItem;
		return this.tail;
	}
	
	public function clone() : LinkedList
	{
		var list : LinkedList = new LinkedList();
		var i : Iterator = iterator();
		
		while( i.hasNext() )
		  list.addItem( i.getNext() );

		return list;
	}
	
	public function getFirst()
	{
		return this.head;
	}
	
	public function getLast()
	{
		return this.tail;
	}
	
	public function getIndexOfData( object ) : Number
	{
		var index:Number = 0;
		var listItem : LinkedListItem = this.head;
		
		for( index = 0; listItem != undefined; index++ )
		{
			if( getComparator().equal( listItem.data, object ) )
				return index;
				
 			listItem = listItem.next;
		}
		
		return -1;
	}
	
	public function getLastIndexOfData( object ) : Number
	{
		var foundIndex : Number = -1;
		var listItem : LinkedListItem = this.head;
		
		for( var index = 0; listItem != undefined; index++ )
		{
			if( getComparator().equal( listItem.data, object ) )
				return index;
				
			listItem = listItem.next;
		}
		
		return -1;
	}
	
	public function removeFirst()
	{
		return removeItem( getFirst() );
	}
	
	public function removeLast()
	{
		return removeItem( getLast() );
	}
	
	public function setDataAt( index : Number, object )
	{
		var listItem:LinkedListItem = this.getItemAt( index );
		var data = listItem.data;
		listItem.data = object;
		return data;
	}
	
	function getItemAt( index : Number ) : LinkedListItem
	{
		var curItem = head;
		
		for( var i = 0; i != index; i++ )
		  curItem = curItem.next;
		  
	    return curItem;
	}
}