/**
* @interface 	IExternalData
* This interface should be implemented by classes that loads external data files and parses them a Objects,
* for example a class that loads an XML file and parses it as an Object (XML2Object)
* 
* @author		Federico Campo Piombi [federicocp@hotmail.com]
* @version		1.1
* @date 		13 Nov 2006
*/

/**
* @ignore
* MODIFICATION HISTORY:
* 
* v1.1 - 14 Nov 2006
* Added the onLoadError event
*/

interface com.fwa.system.gateway.IExternalData {
	
	//required: private var _data:Object //this property should store the loaded data
	
	/**
	* This method should empty the _data Object
	* @return	Void
	*/
	public function clear():Void 
	
	/**
	* This method should load the external content 
	* @param	inFilename (String) File to load
	* @return	Void
	*/
	public function load(inFilename:String):Void
	
	/**
	* This method should be used as a callback to register the end of the load process
	* @param	data (Object) The data object loaded
	* @return 	Void
	*/
	public function onLoad(data:Object):Void
	
	/**
	* This method should be used as a callback to register an error in the load process
	* @param	error (String) A message with the error occured
	* @return 	Void
	*/
	public function onLoadError(error:String):Void
	
	/**
	* This getter should retrieve the content of the _data object property
	* @return	Object
	*/
	//public function get data():Object
}