import com.fwa.debug.Tracer;
import com.fwa.events.ListenerBroadcaster;
import com.fwa.events.IListenable;
import mx.utils.Delegate;

/**
* @class 	Window
* @tooltip 	No description available.
* @author 	Federico Campo Piombi
* @version 	1.1
* @date		01 Nov 2006
* 
* @param 	window_scope (MovieClip) scope where the container Window will be created
* @param 	window_name (String) name of the container Window
* @param 	window_depth (Number) depth of the container Window
* @param 	modal (Boolean) wether it uses a modal background or not
*/
class com.fwa.ui.Window implements IListenable{
// properties:
	private var Broadcaster:ListenerBroadcaster; //ListenerBroadcaster Instance
	
	private var modal:Boolean; 				//Stores the Modal value of the window (true/false)
	private var x:Number; 					//Stores the x position of the Window
	private var y:Number; 					//Stores the y position of the Window
	
	private var clip_container:MovieClip;	//reference to the main container 
	private var clip_modal:MovieClip;		//reference to the Modal movieclip
	private var clip_content:MovieClip;		//reference to the Content movieclip
	
	private var modal_identifier:String = "clip_modal"
	private var modal_depth:Number = 10
	private var content_depth:Number = 12
	
	public var destroy_int:Number


//constructor: 
	public function Window(window_scope:MovieClip, window_name:String, window_depth:Number, modal:Boolean) {
		// constructor
		Broadcaster = new ListenerBroadcaster()
		
		var ref = this
		//validate params
		window_scope = (window_scope==undefined || window_scope == null)? _root : window_scope
		window_depth = (window_depth == undefined || isNaN(window_depth))? window_scope.getNextHighestDepth() : window_depth
		window_name  = (window_name=="" || window_name == undefined)? "window_"+window_depth : window_name
		
		//create clips
		addContainerClip(window_scope, window_name, window_depth)
		addContentClip()
		if(modal == true){
			addModalClip()
		}
	}

// private methods:

	/**
	* @method 	addContainerClip
	* @tooltip 	Creates an empty clip to use as container for the window
	* 		   	and defines methods and properites that references to this Window
	* @param 	window_scope (MovieClip)
	* @param 	window_name  (String)
	* @param 	window_depth (Number)
	* @return 	Void
	*/
	private function addContainerClip(window_scope:MovieClip, window_name:String, window_depth:Number):Void {
		clip_container = window_scope.createEmptyMovieClip(window_name, window_depth)
		clip_container.close = Delegate.create(this, close)
		clip_container.show = Delegate.create(this, show)
		clip_container.hide = Delegate.create(this, hide)
		clip_container._window = this
	}
	
	/**
	* @method 	addContentClip
	* @tooltip 	Creates an empty clip to use as content for the window
	* @return 	Void
	*/
	private function addContentClip():Void {
		clip_content = clip_container.createEmptyMovieClip("content",content_depth)
	}
	
	private function debug(method:String, error:String):Void {
		//Tracer.output(error,method,"Window")
	}

	
// public methods:
	/**
	* @method 	addModalClip
	* @tooltip 	adds a Modal clip to the Window (attachs from library)
	* @return 	Void
	*/
	public function addModalClip():Void {
		clip_modal = clip_container.attachMovie(modal_identifier,"modal",modal_depth) //createEmptyMovieClip("modal",1)
		Tracer.output("modal clip:"+clip_modal,"addModalClip", "Window")
		modal = true
	}

	/**
	* @method 	associateContent
	* @tooltip 	Associates an existing MovieClip to the current Window
	* @param 	clip (MovieClip) MovieClip reference
	* @return 	Void
	*/
	public function associateContent(clip:MovieClip):Void {
		clip_content = clip
	}

	/**
	* @method 	attachContent
	* @tooltip 	Attachs a given linkage to the current Window
	* @param 	identifier (String) linkage identifier
	* @return 	Void
	*/
	public function attachContent(identifier:String):Void {
		clip_content = clip_container.attachMovie(identifier, "content", content_depth)
	}

	/**
	* @method 	loadContent
	* @tooltip 	Loads external content in the current Window
	* 		   	Broadcaster passes a reference to the container window, instead of the content window (the one being loaded)
	* 		   	This is useful because the Container clip has methods and properties that references to its Window Class (show, hide, close, _window)
	* @param 	filename (String) file to load
	* @return 	Void
	*/
	public function loadContent(filename:String):Void {
		if(filename!=="" && filename!==undefined){
			var ref = this
			var mcl:MovieClipLoader = new MovieClipLoader()
			var mcl_lst:Object = new Object()
			mcl_lst.onLoadStart = function(tmc:MovieClip){
				ref.Broadcaster.broadcastMessage("onLoadStart",[tmc._parent])
			}
			mcl_lst.onLoadProgress = function(tmc:MovieClip, bl:Number, bt:Number){
				ref.Broadcaster.broadcastMessage("onLoadProgress",[tmc._parent,bl,bt])
			}
			mcl_lst.onLoadComplete = function(tmc:MovieClip){
				ref.Broadcaster.broadcastMessage("onLoadComplete",[tmc._parent])
			}
			mcl_lst.onLoadInit = function(tmc:MovieClip){
				ref.Broadcaster.broadcastMessage("onLoadInit",[tmc._parent])
			}
			mcl_lst.onLoadError = function(tmc:MovieClip, error, httpstatus){
				ref.Broadcaster.broadcastMessage("onLoadError",[tmc._parent,error,httpstatus])
			}
			mcl.addListener(mcl_lst)
			mcl.loadClip(filename, clip_content)
			debug("LoadContent", "loading:"+filename)
		}else{
			debug("LoadContent", "Invalid Filename")
		}
	}

	/**
	* @method 		close
	* @tooltip 		Deletes this window and removes the Container/Modal clips
	* @description	All listeres receiving the "onClose" event, will still receive this Window's reference
	* 				because this method removes the Window after broadcasting the "onClose" message
	* @return 		Void
	*/
	public function close():Void {
		debug("close","Closing Window:"+clip_container._name)
		Broadcaster.broadcastMessage("onClose",[this])
		clip_container.removeMovieClip()
		delete this
	}

	/**
	* @method 	getBounds
	* @tooltip 	Returns this window's bounds {left:Number, right:Number, top:Number, bottom:Number}
	* @return 	Object
	*/
	public function getBounds():Object {
		var bounds:Object = new Object()
		bounds.left = clip_container._x
		bounds.top = clip_container._y
		bounds.right = clip_container._x + clip_container._width
		bounds.bottom= clip_container._y + clip_container._height
		return( bounds )
	}

	/**
	* @method 	getContentClip
	* @tooltip 	Returns a reference to the content clip of this window (Graphical content)
	* @return 	MovieClip
	*/
	public function getContentClip():MovieClip {
		return( clip_content )
	}

	/**
	* @method 	getContainerClip
	* @tooltip 	Returns a reference to the container clip of this window (Graphical Window)
	* @return 	MovieClip
	*/
	public function getContainerClip():MovieClip {
		return( clip_container )
	}

	/**
	* @method 	getModalClip
	* @tooltip 	Returns a reference to the Modal clip of this window
	* @return 	MovieClip
	*/
	public function getModalClip():MovieClip {
		return( clip_modal )
	}

	/**
	* @method 	removeModalClip
	* @tooltip 	Removes the modal clip from the window
	* @return 	Void
	*/
	public function removeModalClip():Void {
		clip_modal.removeMovieClip()
		modal = false
	}

	/**
	* @method 	show
	* @tooltip 	Makes this window Visible
	* @return 	Void
	*/
	public function show():Void {
		clip_container._visible = true
		Broadcaster.broadcastMessage("onShow",[this])
	}
	
	/**
	* @method 	hide
	* @tooltip 	Makes this window invisible
	* @return 	Void
	*/
	public function hide():Void {
		clip_container._visible = false
		Broadcaster.broadcastMessage("onHide",[this])
	}

// listener broadcaster integration:
	/**
	* @method 	addListener
	* @tooltip 	adds a new listener to the Broadcaster's collection
	* @param 	listener_obj (Object) object to add to the Collection
	* @return 	Boolean
	*/
	public function addListener(listener_obj:Object):Boolean {
		return( Broadcaster.addListener(listener_obj) )
	}

	/**
	* @method 	removeListener
	* @tooltip 	removes a listener from the Broadcaster's collection
	* @param 	listener_obj (Object) listener to be removed
	* @return 	Boolean
	*/
	public function removeListener(listener_obj:Object):Boolean {
		return( Broadcaster.removeListener(listener_obj) )
	}


// getter-setters:
	
	/**
	* @method 	_x
	* @tooltip 	Gets/Sets the "x" value of the window
	*/
	public function get _x():Number { return clip_container._x; }
	public function set _x(x:Number):Void { clip_container._x = x; }

	/**
	* @method 	_y
	* @tooltip 	Gets/Sets the "y" value of the window
	*/
	public function get _y():Number { return clip_container._y; }
	public function set _y(y:Number):Void { clip_container._y = y; }
	
	/**
	* @method 	_modal
	* @tooltip 	(getter) Returns the modal value of the window
	*/
	public function get _modal():Boolean { return modal; }
	//public function set _modal(val:Boolean):Void { modal = val; }

	/**
	* @method 	_visible
	* @tooltip 	(getter) Returns the "visible" value of the Window
	*/
	public function get _visible():Boolean { return clip_container._visible; }
	//public function set _visible(val:Boolean):Void { clip_content._visible = val; }
	
	/**
	* @method 	_name
	* @tooltip 	(getter) Returns the name of the Window
	*/
	public function get _name():String { return clip_container._name; }

	
// events:

	/**
	* @event onClose [Broadcasted]
	*/

	/**
	* @event onShow [Broadcasted]
	*/

	/**
	* @event onHide [Broadcasted]
	*/

	/**
	* @event onLoadComplete [Broadcasted]
	*/

	/**
	* @event onLoadInit [Broadcasted]
	*/

	/**
	* @event onLoadProgress [Broadcasted]
	*/

	/**
	* @event onLoadStart [Broadcasted]
	*/
}

// ** END Window ***************************************************