/**
* This class provides methods to display and arrange the TooltipBox
* @version	1.0
* @date 	30 Nov 2006
*/
class com.fwamarco.ui.TooltipBox extends MovieClip {
	
	//layout elements
	private var clip_bg:MovieClip;
	private var clip_flechita:MovieClip;
	private var t_tooltip:TextField;
	private var alphaTo:Function;
	
	//properties
	private var font_size:Number;
	private var font_color:Number;
	
	
	/**
	* Constructor
	*/
	public function TooltipBox(){
		//set default style
		font_size = 10;
		font_color = 0x000000;
		
		//set textfield properties
		t_tooltip.wordWrap = true
		t_tooltip.autoSize = true
		t_tooltip.html = true
		
		//hide the box
		hide();
	}
	
	/**
	* Displays the Tooltip Box with the given text, and calculates the display direction of the Tooltip Box
	* @param	txt (String) text to display
	* @return	Void
	*/
	public function show(txt:String):Void {
		setText(txt)
		//determinar para que lado se abre
		var t_h:Number = clip_bg._height+clip_flechita._height
		if(this._y+t_h > Stage.height){ alignTop() }else{ alignBottom()	}
		if(this._x+clip_bg._width > Stage.width){ alignLeft() }else{ alignRight() }
		this._alpha = 0
		this._visible = true
		this.alphaTo(100, 0.5, "easeOutQuint", 0)
	}
	
	/**
	* Hides the Tooltip Box
	* @return	Void
	*/
	public function hide():Void {
		this._visible = false
	}
	
	/**
	* Sets the Tooltip Box's text and resizes the background to fit
	* @param	t (String) text to display
	* @return	Void
	*/
	private function setText(t:String):Void {
		var tf:TextFormat = new TextFormat()
		tf.size = font_size
		tf.color = font_color
		t_tooltip.setNewTextFormat(tf)
		t_tooltip.text = t
		
		clip_bg.setSize(t_tooltip.textHeight+10)
	}
	
	/**
	* Aligns the Tooltip Box vertically on top of the registration point
	* @return	Void
	*/
	private function alignTop():Void {
		clip_flechita._yscale = 100
		clip_flechita._y = Math.floor(clip_flechita._height)*-1
		clip_bg._y = (clip_flechita._y+2) - Math.floor(clip_bg._height)
		var bg_middle:Number = clip_bg._y + (clip_bg._height/2)
		t_tooltip._y = bg_middle -(t_tooltip._height/2)
	}
	
	/**
	* Aligns the Tooltip Box vertically at the bottom of the registration point
	* @return	Void
	*/
	private function alignBottom():Void {
		clip_flechita._yscale = -100
		clip_flechita._y = Math.ceil(clip_flechita._height)
		clip_bg._y = (clip_flechita._y-3)
		var bg_middle:Number = clip_bg._y + (clip_bg._height/2)
		t_tooltip._y = bg_middle -(t_tooltip._height/2)
	}

	/**
	* Aligns the Tooltip Box horizontally at the right of the registration point
	* @return	Void
	*/
	private function alignRight():Void {
		clip_bg._x = -20
		t_tooltip._x = -10
	}
	
	/**
	* Aligns the Tooltip Box horizontally at the left of the registration point
	* @return	Void
	*/
	private function alignLeft():Void {
		clip_bg._x = -140
		t_tooltip._x = -130
	}
	
	/**
	* Sets the TooltipBox style
	* @param	style (Object) Object that contains the Style values
	* @return	Void
	*/
	public function setStyle(style:Object):Void {
		font_size = (style.font_size!==undefined)? style.font_size : font_size
		font_color = (style.font_color!==undefined)? style.font_color : font_color
		clip_bg.setStyle(style.background_color, style.background_alpha)
	}
}