/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.iterator.*;

class asdk.HashMap extends AbstractMap
{
  private var table : Object;
  private var hashes : Vector;
  private var count : Number;
  
  function HashMap( map : Map )
  {		
  	table = new Object();
	hashes = new Vector();
	count = 0;
	
	if( map != undefined )
	  putAll( map );
  }
  
  function getSize() : Number
  {
    return count;
  }

  function get( key )
  {
    if( key == undefined )
	{
		throw new Error( "cannot lookup null values" );
		return null;
	}
		
	var tab : Object = table;
	var hash : String = StringUtils.getHashCode( key );
	
	if( table[ hash ] == undefined )
		return null;
	
	var ll : LinkedList = table[ hash ];
	var llItem : LinkedListItem = ll.getFirst();
	var foundLLItem : LinkedListItem;
	
	while( llItem != undefined )
	{
		var foundKey = llItem.data.key;
		
		if( getComparator().execute( foundKey, key ) )
		{
			foundLLItem = llItem;
			break;
		}
		
		llItem = llItem.next;
	}
	
	if( foundLLItem != undefined )
		return foundLLItem.data.value;		
  }
  
  function put( key, value )
  {
    if( value == undefined )
	{
		throw new Error( "cannot put null keys" );
		return;
	}
		
	var tab : Object = table;
	var hash : String = StringUtils.getHashCode( key );
	
	// if the hash is not in the hashtable at all;
	if( table[ hash ] == undefined )
	{
		table[ hash ] = new LinkedList();
		var item = { key:key, value:value, hash:hash };
		table[ hash ].addItem( item );
		hashes.addItem( hash );
		count++;
		return value;
	}
	
	// if the hash is in the hashtable, check for equality between key and hash values
	var ll : LinkedList = table[ hash ];
	var llItem : LinkedListItem = ll.getFirst();
	var keyExists;
	var foundHash : String;
	var foundValue;
	var foundLLItem : LinkedListItem;
	
	while( llItem != undefined )
	{
		var llKey = llItem.getData().key;
		
		if( getComparator().execute( key, llKey ) )
		{
			foundLLItem = llItem;
			break;
		}
		
		llItem = llItem.next;
	}
	
	// if there is already a key in the entry that matches the key passed in.
	if( foundLLItem != undefined )
	{
		var dataObj = foundLLItem.getData();
		var oldValue = dataObj.value;
		dataObj.value = value;
		return oldValue;
	}
	else
	{
		var newItem : Object = { key:key, hash:hash, value:value }
		ll.addItem( newItem );
		hashes.addItem( hash );
		count++;
		return value;
	}
  }
  
  function remove( key )
  {
	  var hash = StringUtils.getHashCode( key );
	  var ll : LinkedList = table[ hash ]
	  
	  if( ll == undefined )      
	  	return;
	  
	  var llItem : LinkedListItem = ll.getFirst();
	  var foundItem : LinkedListItem;
	  
	  while( llItem != undefined )
	  {
		  var foundHash : String = llItem.data.hash;
		  var foundKey = llItem.data.key;
		  
		  if( foundHash == hash && getComparator().execute( key, foundKey ) )
		  {
			  foundItem = llItem;
			  break;
		  }														
		  
		  llItem = llItem.next;
	  }
	  
	  if( foundItem != undefined )
	  {
		  var oldValue = foundItem.data.value;
		  ll.removeItem( foundItem );
    	  hashes.removeItem( hash );
		  count--;
		  
		  if( ll.getFirst() == undefined )
		    delete table[ hash ];
			
		  return oldValue;
	  }
  }
    
  function getKeys() : Set
  {
    return keySet();
  }
  
  function keySet() : Set
  {
	  return new KeySet( this );
  }
  
  function getValues() : Collection
  {
	  return new ValueCollection( this );
  }
  
  function iterator( type : Number )
  {
	  return new HashMapIterator( hashes, table, this, type == ASDKConstants.KEYS );
  }
}