import com.fwaapi.buttons.KeyButton;
import com.fwaapi.Tooltip;
import mx.utils.Delegate;

/**
* This class extends the AbstracButton's functionality by allowing the user to
* set a Tooltip message to display on RollOver.
* 
* @version	1.2
* @since 	26 Dec 2006
*
* 
* @ignore
* MODIFICATION HISTORY
* 
* 1.1 - 11 Ene 07
* + TooltipButton now extends KeyButton (previously extended AbstractButton)
* 
* 1.2 - 06 Feb 07
* + Added "tooltipEnabled" getter-setter and the
*   possibility to disable tooltip display at runtime
* + Added "tooltip" getter
*/
class com.fwaapi.buttons.TooltipButton extends KeyButton {
	
	//properties
	private var _tooltip:String;
	private var _enabled:Boolean = true;
	
	//getter-setters
	
	/**
	* Getter-Setter: Sets the Tooltip text for the button
	* @param	val (String) Tooltip text
	* @return	Void
	*/
	public function set tooltip(val:String):Void {
		_tooltip = val;
	}
	public function get tooltip():String {
		return _tooltip
	}
	
	/**
	* Enables/disables the Button to display the tooltip
	* @param	val (Boolean) true if the Button should display the Tooltip. Otherwise false
	* @return	Boolean
	*/
	public function set tooltipEnabled(val:Boolean):Void {
		_enabled = val
	}
	public function get tooltipEnabled():Boolean {
		return _enabled
	}
	
	
	/**
	* Constructor
	*/
	public function TooltipButton(){
		this.onRollOver = Delegate.create(this, displayTooltip);
		this.onRollOut = Delegate.create(this, hideTooltip);
		this.onReleaseOutside = Delegate.create(this, hideTooltip);
		this.onRelease = Delegate.create(this, hideTooltip);
	}
	
	
	/**
	* Displays the user defined Tooltip (if any)
	* @return	Void
	*/
	private function displayTooltip():Void {
		if(_enabled && _tooltip!=="" && _tooltip!==undefined){
			Tooltip.show(_tooltip);
		}
	}
	
	/**
	* Hides the displayed Tooltip
	* @return	Void
	*/
	private function hideTooltip():Void {
		Tooltip.hide();
	}
}