import com.fwamarco.consoles.IConsole;
import com.fwamarco.consoles.ConsolesManager;

/**
* This class provides the implementation of an Abstract Console to be extended by all other Consoles.
* This way, the other consoles classes can focus in the console functionality itself, 
* rather than in the integration with the ConsolesManager code.
* 
* @version	1.0
* @date 	27 Nov 2006
*/
class com.fwamarco.consoles.AbstractConsole extends MovieClip implements IConsole {
	
	private var _instanceName:String;
	private var clip_bg:MovieClip;
	
	/**
	* Constructor.
	* Registers this Console in the ConsolesManager.
	* The console is registered under its instance name
	*/
	public function AbstractConsole(){
		//register console
		_instanceName = this._name
		ConsolesManager.registerConsole(_instanceName, this)
		
		//disable background
		clip_bg.onRelease = function(){}
		clip_bg.useHandCursor = false
		clip_bg.enabled = false
		
		//hide the console
		this._visible = false
	}
	
	/**
	* Displays the console
	* @return	Void
	*/
	public function show():Void {
		if(this._visible == false){
			ConsolesManager.closeAll()
			this._visible = true
		}
	}
	
	/**
	* Hides the console
	* @return	Void
	*/
	public function hide():Void {
		if(this._visible == true){
			this._visible = false
			this.onClose()
		}
	}
	
	
	//events
	/**
	* Eevent executed when the console is hidden
	*/
	public function onClose():Void {}
}