import com.fwamarco.ui.LangSelector;
import com.fwaapi.SystemProxy;
import com.fwaapi.Application;
import com.fwaapi.Tooltip;
import mx.utils.Delegate;

/**
* Document class for the Marco file.
* Initialises several marco and System assets
* 
* @version	2.0
* @since	17 Mar 2007
*/
class com.fwamarco.ui.Document extends MovieClip {
	//PROPERTIES
	private var clip_tooltip:MovieClip;
	private var clip_marco:MovieClip;
	private var clip_loader:MovieClip;
	private var clip_lang:MovieClip;
	
	
	/**
	* Static method that enables to set up the Documen Class 
	* in AS2.
	* @param	target (MovieClip) Clip to set as Document
	*/
    public static function main(target:MovieClip):Void {
        target.__proto__ = Document.prototype;
        target.init();
    }
    
	
	/**
	* Initialises the Marco configuration
	*/
    private function init():Void {
		//initialize LangSelector
		var ref:Document = this;
		clip_lang = this.attachMovie("clip_langselector", "lang", this.getNextHighestDepth());
		clip_lang._x = Math.round(Stage.width/2);
		clip_lang._y = Math.round(Stage.height/2);
		
		//register to LangSelector events
		var langSelector:LangSelector = LangSelector(clip_lang);
		langSelector.onLanguageSelected = Delegate.create(this, this.initSystemLanguage);
		
		
		//register to System events
		SystemProxy.onSystemBegin = function(){
			ref.clip_loader.show()
		}
		SystemProxy.onSystemProgress = function(overallPercent:Number){
			ref.clip_loader.setProgress(overallPercent)
		}
		SystemProxy.onSystemStartup = function(success:Boolean, error:String):Void {
			if(success){
				ref.clip_loader.hide();
			}else{
				trace("SYSTEM INITIALISATION ERROR");
			}
		}
		//SystemProxy.onSystemStartup = Delegate.create(this, initialiseMarco)
		SystemProxy.onLanguageStartup = Delegate.create(this, onLanguageStartup);
		
		
		//register to Application events
		var app_lst:Object = new Object()
		app_lst.onLoadStart = function(){
			ref.clip_loader.show();
		}
		app_lst.onLoadInit = function(){
			ref.clip_loader.hide()
		}
		app_lst.onLoadProgress = function(tmc:MovieClip, bl:Number, bt:Number){
			ref.clip_loader.setProgress(Math.ceil(bl*100/bt))
		}
		Application.setListener(app_lst)
    }
	
	
	/**
	* Initialises the System with the selected language configuration.
	* Once the System is initialized, executes the Marco initialisation
	* and application preloading.
	* 
	* @param	langFolder (String) Language folder for the application
	* @param	langFile (String) Language file for the Marco and apps list
	*/
	private function initSystemLanguage(langFolder:String, langFile:String):Void {
		//attach loader
		clip_loader = this.attachMovie("ApplicationPreloader", "loader", this.getNextHighestDepth())
		clip_loader._x = Math.round(Stage.width/2);
		clip_loader._y = Math.round(Stage.height/2);
		
		SystemProxy.initLanguage(langFolder, langFile);
	}
	
	
	/**
	* Event executed when the language file was loaded and parsed OK.
	* Attaches the Marco and initialises its language elements.
	* Attaches the Tooltip Clip and registers it in the Tooltip API.
	* 
	* @param	success (Boolean) true if the language file was loaded
	*/
	private function onLanguageStartup(success:Boolean):Void {
		if(success){
			//remove lang clip
			clip_lang.removeMovieClip()
			
			//init marco
			clip_marco.removeMovieClip()
			clip_marco = this.attachMovie("_MARCO", "marco", this.getNextHighestDepth())
			clip_marco._x = clip_marco._y = 0
			clip_marco.init()
			
			//register tooltip
			clip_tooltip = this.attachMovie("TooltipBox", "tooltip", this.getNextHighestDepth())
			Tooltip.registerClip(clip_tooltip)
			
			//init default app
			SystemProxy.initApplication(0);
		}else{
			trace("LANGUAGE STARTUP ERROR: Aborting initialisation")
		}
	}
}

