import com.parker.users.User;
import com.parker.core.URLs;
import com.parker.core.Configuration;
import com.parker.content.WindowsProxy;
import com.wunderman.forms.FormsProcessor;
import com.wunderman.utils.XMLConstruct;
import com.wunderman.debug.Environment;
import mx.utils.Delegate;

/**
* @class 	Registration
* @tooltip 	
* @author	Federico Campo Piombi
* @version 	1.0
* @date 	07 Nov 2006
*/
class com.parker.forms.Registration extends MovieClip {
	//properties
	private var fp:FormsProcessor;
	private var wproxy:WindowsProxy;
	
	//form fields
	private var i_nombre:TextField;
	private var i_apellido:TextField;
	private var i_dni:TextField;
	private var i_telefono:TextField;
	private var i_email:TextField;
	private var i_pass:TextField;
	private var i_passconfirm:TextField;
	private var i_ciudad:TextField;
	private var cb_dia:MovieClip;
	private var cb_mes:MovieClip;
	private var cb_anio:MovieClip; 
	private var cb_pais:MovieClip;
	private var cb_destino:MovieClip;
	private var bt_send:Button;
	
	/**
	* @constructor
	*/
	public function Registration(){
		fp = new FormsProcessor();
		wproxy = WindowsProxy.getInstance();
	}
	
	//public methods
	
	/**
	* @method 	init
	* @tooltip 	Initialises form variables
	* @return 	Void
	*/
	public function init():Void {
		initProcessor()
		fillCombos()
		bt_send.onRelease = Delegate.create(this, sendForm)
	}
	
	
	//private methods
	
	/**
	* @method 	sendForm
	* @tooltip 	Submits the registration Form
	* @return	Void
	*/
	private function sendForm():Void {
		if(validateForm()){
			var ref = this
			var send_lv:LoadVars = new LoadVars();
			var receive_lv:LoadVars = new LoadVars();
			
			//definir el receiver
			receive_lv.onLoad = function(){
				trace("REGISTRATION RECEIPT:"+unescape(this))
				var parsed:Object = ref.parseResponse(unescape(this))
				switch(parsed.error._value){
					case "0":
						User.login(parsed)
						var cback:Object = new Object()
						cback.onClose = Delegate.create(ref, ref.onRegistrationFinished)
						ref.wproxy.openAlert("FELICITACIONES!", "Te registraste con xito", "alert", cback)
						break;
					case "1":
						ref.wproxy.openAlert("ATENCION!", "Ya existe un usuario registrado\ncon ese e-mail", "alert");
						break;
					default:
						ref.wproxy.openAlert("ATENCION!", "Error desconocido: "+parsed.error._value, "alert");
						break;
				}
			}
			
			//definir el sender
			send_lv.nombre = i_nombre.text;
			send_lv.apellido = i_apellido.text;
			send_lv.documento = i_dni.text;
			send_lv.email = i_email.text;
			send_lv.contrasena = i_pass.text;
			send_lv.telefono = i_telefono.text;
			send_lv.ciudad = i_ciudad.text;
			send_lv.pais = cb_pais.getValue();
			send_lv.destino_elegido = cb_destino.getValue();
			send_lv.ano_nacimiento = cb_anio.getValue()
			send_lv.dia_nacimiento = cb_dia.getValue()
			send_lv.mes_nacimiento = cb_mes.getValue()
			send_lv.sexo = "M"
			
			//enviar
			/*
			if(Environment.isTestMovie()){
				receive_xml.load(URLs.registration)
			}else{
			*/
			send_lv.sendAndLoad(URLs.registration, receive_lv, "POST")
			//}
			
		}
	}
	
	/**
	* @method 	validateForm
	* @tooltip 	Validates the form input and returns TRUE if it's ok
	* @return	Boolean
	*/
	private function validateForm():Boolean {
		if(fp.isValid()==true){
			var inner_errors:Array = new Array()
			
			//validar que matchee la confirmacin de pass
			if(i_pass.text !== i_passconfirm.text){
				inner_errors.push("Password y Confirmacin no coinciden!")
			}
			
			//validar que no haya errores internos
			if(inner_errors.length==0){
				return(true);
			}else{
				wproxy.openAlert("ATENCION!", inner_errors[0], "alert");
				return(false)
			}
		}else{
			var err:Object = fp.getError()
			var err_msg:String = (err.type!=="combo" && err.type!=="checkbox")? err.name : err.error
			wproxy.openAlert("ATENCION!", err_msg, "alert");
			return(false);
		}
	}
	
	
	/**
	* @method 	initProcessor
	* @tooltip 	Registers the Form's fields into the FormsProcessor
	* @return	Void
	*/
	private function initProcessor():Void {
		fp.addField(i_nombre, "text","Debes ingresar tu nombre","A-Z a-z", 50);
		fp.addField(i_apellido, "text", "Debes ingresar tu apellido", "A-Z a-z", 50);
		fp.addField(i_dni, "number", "Debes ingresar tu DNI", "0-9", 10);
		fp.addField(i_telefono, "text", "Debes ingresar tu telfono", "0-9.-() ", 25);
		fp.addField(i_ciudad, "text", "Debes ingresar tu Ciudad", "", 150);
		fp.addField(i_email, "email", "E-Mail invlido", "A-Z0-9a-z@._\\-", 150);
		fp.addField(i_pass,"text","Password invlido","",150);
		fp.addField(i_passconfirm,"text","Debes confirmar el password","",150);
		
		fp.addCombo(cb_anio, -1, "Debes seleccionar el ao de nacimiento");
		fp.addCombo(cb_mes, -1, "Debes seleccionar el mes de nacimiento");
		fp.addCombo(cb_dia, -1, "Debes seleccionar el da de nacimiento");
		fp.addCombo(cb_pais, -1, "Debes seleccionar tu pais de residencia");
		fp.addCombo(cb_destino, -1, "Debes seleccionar un destino!");
		
		fp.setDefaultErrorMessage("Debes completar todos los campos obligatorios")
		fp.clearFields()
	}
	
	/**
	* @method 	fillCombos
	* @tooltip 	Loads the data into the combos
	* @return 	Void
	*/
	private function fillCombos():Void {
		//fill dates
		for(var i:Number = 1; i<=31; i++){
			cb_dia.addItem(i, i)
		}
		
		for(var i:Number = 1; i<=12; i++){
			cb_mes.addItem(i,i)
		}
		
		var tdate:Date = new Date();
		var tyear:Number = tdate.getFullYear();
		for(var i:Number = tyear; i>tyear-100; i--){
			cb_anio.addItem(i, i)
		}
		
		var paises:Object = Configuration.getPaises().pais
		var pcount:Number = paises.length
		for(var i:Number=0; i<pcount; i++){
			var tpais:Object = paises[i]
			cb_pais.addItem(tpais.nombre._value, tpais.id._value)
		}
		
		var destinos:Object = Configuration.getDestinos().destino
		var dcount:Number = destinos.length
		for(var i:Number=0; i<dcount; i++){
			cb_destino.addItem(destinos[i].nombre._value, destinos[i].id._value)
		}
	}
	
	/**
	* @method 	parseResponse
	* @tooltip 	Converts a LoadVars XML-like response into an object
	* @param	str (String) LoadVars receipt
	* @return	Object
	*/
	private function parseResponse(str:String):Object {
		trace("str: "+str)
		var xmlstr:String = str.substring(0, str.indexOf("="))
		xmlstr = xmlstr.split("\n").join("")
		trace("xmlstr: "+xmlstr)
		var tXML:XML = new XML(xmlstr)
		tXML.ignoreWhite = true
		trace("xml: "+tXML)
		var oXML:Object = new XMLConstruct(tXML)
		return (oXML)
	}
	
	
	//events
	/**
	* @event	onRegistrationFinished
	* @tooltip 	Event executed when the user has finished his Registration with no errors
	*/
	private function onRegistrationFinished():Void {
		_parent.gotoAndPlay("compra")
	}
}