/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.condition.*;

class asdk.algorithm.Filtering
{
    static function exclude( collection : Collection, condition : UnaryCondition ) : Collection
    {
        var destination : Collection = collection.New();
        var iterator : Iterator = collection.iterator();

        while( iterator.hasNext() )
        {
            var object = iterator.getNext();

            if( !condition.execute( object ) )
              destination.addItem( object );
        }

        return destination;
    }

    static function include( collection : Collection, condition : UnaryCondition ) : Collection
    {
        var destination : Collection = collection.New();
        var iterator : Iterator = collection.iterator();

        while( iterator.hasNext() )
        {
            var object = iterator.getNext();

            if( condition.execute( object ) )
              destination.addItem( object );
        }

        return destination;
    }

    static function unique( collection : Collection, comparator : IComparator ) : Collection
    {
        if( comparator == undefined )
          comparator = new ObjectComparator();

        var hashset : HashSet = new HashSet();
        hashset.setComparator( comparator );
        var iterator : Iterator = collection.iterator();

        while( iterator.hasNext() )
        {
            var object = iterator.getNext();

            if( !hashset.containsItem( object ) )
                hashset.addItem( object );
        }

        var destination : Collection = collection.New();
        destination.addAll( hashset );
        return destination;
    }

    static function uniqueConsecutive( collection : Collection, condition : BinaryCondition ) : Collection
    {
        if( condition == undefined )
            condition = new ObjectsEqual();

        var hashset : HashSet = new HashSet();
        var iterator : Iterator = collection.iterator();
        var lastObject;

        while( iterator.hasNext() )
        {
            var curObject = iterator.getNext();

            if( condition.execute( curObject, lastObject ) && !hashset.containsItem( curObject ))
              hashset.addItem( curObject );

            lastObject = curObject;
        }

        var destination : Collection = collection.New();
        destination.addAll( hashset );
        return destination;
    }
}
