/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.xml.*;

class asdk.xml.Element extends asdk.xml.Parent
{
    var prefix:String;
    var namespace:String;
    var name:String;
    var attributes:NodeList;
    
    public function Element( elementName, textString )
    {
       super();    
       
       if( elementName != null )
          setName( elementName );
       
       if( textString != null )
          addChild( new Text( textString ) );
    }
    
    public function parse( xmlNodeObj:XMLNode )
    {
        this.attributes = new NodeList();

        // process attributes first, so we can resolve namespace if needed
        for( var attribName in xmlNodeObj.attributes )
        {
            var attribute:Attribute = new Attribute( attribName, xmlNodeObj.attributes[ attribName ] );
            attributes.append( attribute );
        }

        this.name = xmlNodeObj.nodeName;
        var colonPrefix = this.name.indexOf( ":" );

        if( colonPrefix > -1 )
        {
            this.prefix = this.name.substr( 0, colonPrefix );
            this.name = this.name.substr( colonPrefix + 1 );
        }

        if( this.prefix == undefined )
            this.namespace = getDefaultNamespace();
        else
            this.namespace = getNamespace( this.prefix );

        if( xmlNodeObj.hasChildNodes() )
            for( var i = 0; i < xmlNodeObj.childNodes.length; i++ )
            {
                var nodeObj:Child = NodeFactory.createNode( xmlNodeObj.childNodes[ i ], this );
                addChild( nodeObj );
            }
    }

    /**
     * Return the root of my parent, or null if I have no parent.
     */
    public function getRoot() : Element
    {
        return (this.parent == null ? this : this.parent.getRoot());
    }

    /**
     * Return my parent element.
     */
    public function getParentElement() : Element
    {
        var parent = getParent();
        var castParent:Element = parent;
        return castParent;
    }

    /**
     * Return my name.
     */
    public function getName() : String
    {
        return name;
    }

    /**
     * Set my name.
     * @param name The new name value.
     * @return Myself, to allow cascading.
     */
    public function setName( name : String ) : Element
    {
        this.name = name;
        this.namespace = getDefaultNamespace();
        return this;
    }

   /**
    * Return true if my namespace and name match the specified values.
    *
    * @param namespace The namespace.
    * @param name      The name.
    */
    public function hasName( namespace, name )
    {
        if( !this.name == name )
          return false;
        else if( namespace == undefined )
          return true;
        else if( this.namespace == undefined )
          return false;
        else
          return this.namespace == namespace;
    }

    /**
     * Return the value of the default namespace, or null if there is none.
     * This operation searches up the element hierarchy starting at this element.
     */
    public function getDefaultNamespace() : String
    {
        return getNamespace( "xmlns" );
    }

    /**
     * Return the value of the namespace with the specified prefix, or null if
     * there is none. If the prefix is the empty string "", return the default
     * namespace. This operation searches up the element hierarchy starting
     * at this element.
     * @param prefix The prefix.
     */
    public function getNamespace( prefix : String ) : String
    {
        if( prefix.length == 0 )
          prefix = "xmlns";

        for( var attribute = attributes.firstNode; attribute != undefined; attribute = attribute.nextNode )
          if( attribute.isNamespace && attribute.name == prefix )
            {
            var value = attribute.value;
            return (value.length > 0 ? value : undefined); // support xmlns=""
            }

        return (parent == undefined ? undefined : parent.getNamespace( prefix ));
    }

    public function getString( name:String, namespace:String) : String
    {
        return getText( name, namespace ).getString();
    }

    public function hasText()
    {
        return (getText() != null);
    }


    // function supports the following formats:
    // 1. getText() - returns the first Text object for this element
    // 2. getText( name ) - returns the first text object for the subelement with name
    // 2. getText( name, namespace ) - returns the first text object for the subelement with name and namespace
    public function getText( name:String, namespace:String ) : Text
    {
        if( name == undefined && namespace == undefined )
        {
            for( var node = children.firstNode; node != null; node = node.nextNode )
               if( node instanceof Text )
                   return node;

            return null;
        }
        else
        {
            var element:Element = getElement( name, namespace );
            return element == null ? null : element.getText();
        }
    }

    public function getAttributes() : NodeList
    {
        return attributes;
    }

    public function getAttribute( name:String ) : Attribute
    {
        for( var attribute = attributes.firstNode; attribute != undefined; attribute = attribute.nextNode )
            if( attribute.name == name )
                return attribute;

        return null;
    }
}
