import com.fwa.debug.Tracer;

/**
* This class allows to create any given number of configuration values.
* Values are stored in a single object, and since the class is Static, can be accessed from anywhere.
* This version does not check for existant values when adding new ones, so if you store a variable that already exists,
* the new one will overwrite the old one.
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.0
* @since	14 Nov 2006
* 
* @ignore 
* TO DO: 
* + add verification to the <store> method to avoid overwriting
* + add a boolean return type to the <store> method (true if var was created, false if already exists)
* + add an <update> method to allow a value to be updated.
* + add a boolean return type to the <update> method (true if updated successfully, false if var doesn't exists)
*/

class com.fwa.system.Configuration {
	
	private static var config:Object = new Object() //object that will store all the created values
	
	/**
	* @constructor
	*/
	private function Configuration() {}
	
	/**
	* This method stores a new value in the Configuration under the given label.
	* The method does not check if the label already exists, so it will allways be overwritten.
	* @param	label (String) Name of the variable to create in the config Object
	* @param	value (Object) Value of the new variable
	* @return	Void
	*/
	public static function store(label:String, value:Object):Void {
		config[label] = value
	}
	
	
	/**
	* This method retrieves any stored value withtin the config object
	* @param	label (String) Name of the variable whose value wants to be retrieved
	* @return	Object
	*/
	public static function retrieve(label):Object {
		return( config[label] )
	}
	
}