import com.fwaapi.Barrido;
import com.fwa.system.FWAConfiguration;
import com.fwamarco.ui.TooltipBox;

/**
* Creates a Tooltip Box and displays the given text in it
* Requires that the "TooltipBox" clip exists to attach it
* @version	1.2
* @since 	30 Nov 2006
* 
* 
* @ignore
* MODIFICATION HISTORY
* 
* 1.1 - 08 Feb 07
* + Added possibility to pass as arguments the X and Y position
*   of the Tooltip Box
* 
* 1.2 - 19 Mar 07
* + Removed X and Y arguments from "show" method
* + TooltipBox position is resolved: if Barrido is enabled, the tooltip
*   is displayed at the current highlighted element X/Y posiiton, otherwise
*   the Mouse position is used to locate the TooltipBox.
*/
class com.fwaapi.Tooltip {
	
	private static var config:Object = FWAConfiguration.main.accesibility.tooltips
	private static var tooltip_clip:MovieClip;
	
	
	/**
	* Constructor
	*/
	private function Tooltip(){}
	
	/**
	* Displays and configures the TooltipBox.
	* Optional X and Y position can be passed to this method. Otherwise Mouse Position is used.
	* @param	text (String) Text to display in the TooltipBox
	* @return	Void
	*/
	public static function show(text:String):Void {
		if(config.enabled._value.toLowerCase() == "true"){
			tooltip_clip._x = (Barrido.enabled)? Barrido.getElement()._x : _root._xmouse;
			tooltip_clip._y = (Barrido.enabled)? Barrido.getElement()._y : _root._ymouse;
			
			var tooltipbox:TooltipBox = TooltipBox(tooltip_clip)
			
			var t_style:Object = new Object()
			t_style.font_color = Number("0x"+config.font.color._value)
			t_style.font_size = Number(config.font.size._value)
			t_style.background_color = Number("0x"+config.background.color._value)
			t_style.background_alpha = Number(config.background.alpha._value)
			
			tooltipbox.setStyle(t_style)
			tooltipbox.show(text)
		}
	}
	
	/**
	* Hides the Tooltip
	* @return	Void
	*/
	public static function hide():Void {
		var tooltipbox:TooltipBox = TooltipBox(tooltip_clip)
		tooltipbox.hide()
	}
	
	/**
	* Sets the clip to use as a TooltipBox
	* @param	t_clip (MovieClip) Tooltip clip reference
	* @return	Void
	*/
	public static function registerClip(t_clip:MovieClip):Void {
		tooltip_clip = t_clip
	}
}