import com.fwaapi.buttons.AbstractButton;
import mx.utils.Delegate;

/**
* This class extends the AbstracButton's functionality by allowing the user to
* configure an Action Key. The Action Key configured will trigger the onRelease
* event of the button.
* 
* @version	1.0
* @since 	10 Jun 2006
*/
class com.fwaapi.buttons.KeyButton extends AbstractButton {
	
	//properties
	private var keyCode:Number;		//stores the Action Key Number
	private var keyType:String;		//defines the Type of the Key ("code" or "ascii")
	private var keyListener:Object;	//stores the Key Listener Object
	
	/**
	* Constructor
	*/
	public function KeyButton(){
		super();
		keyListener = new Object();
		keyListener.onKeyDown = Delegate.create(this, onKeyDown);
		keyListener.onKeyUp = Delegate.create(this, onKeyUp);
	}
	
	/**
	* Sets the Key that will trigger the onRelease of the Button
	* As the stored keyCode is a Number, if a character is provided then 
	* its ascii code will be saved.
	* Also the Key number type is stored: "ascii" if the provided ActionKey is a String
	* (ie: "a"), and "code" if the provided ActionKey is a number (ie: Key.UP)
	* Registers the Key Listener
	* 
	* @param	key (Number / String) action key
	* @return	Void
	*/
	public function setActionKey(key):Void {
		switch (typeof(key)){
			case "number":
				//trace("type number")
				keyCode = key;
				keyType = "code";
				break;
			case "string":
				//trace("type string")
				keyCode = key.charCodeAt(0);
				keyType = "ascii";
				break;
			default:
				removeActionKey();
				break;
		}
		Key.addListener(keyListener);
	}
	
	/**
	* Removes any previously set ActionKey
	* Unregisters the Key Listener.
	*/
	public function removeActionKey():Void {
		keyCode = null;
		keyType = null;
		Key.removeListener(keyListener);
	}
	
	
	/**
	* Gets the currently pressed Key and evaluates if it matchs the Action Key.
	* If the Keys match, the action is executed.
	*/
	private function onKeyUp():Void {
		if(isActionKey()){
			this.onRollOut();
			this.onRelease();
		}
	}
	
	/**
	* Simulates the "rollOver" effect when the ActionKey is down
	*/
	private function onKeyDown():Void {
		if(isActionKey()){
			this.onRollOver();
		}
		
	}
	
	/**
	* Returns true if the currently pressed Key is the Action Key
	* @return	Boolean
	*/
	private function isActionKey():Boolean {
		//trace("ascii:"+Key.getAscii()+" code:"+Key.getCode()+" set:"+keyCode+" type:"+keyType);
		return 	((keyType=="code" && Key.getCode()==keyCode) || (keyType=="ascii" && Key.getAscii()==keyCode));
	}
}