import com.fwamarco.consoles.AbstractConsole;
import com.fwaapi.Barrido;
import com.fwaapi.Language;
import com.fwa.system.FWAConfiguration;
import mx.utils.Delegate;

/**
* This class manages the Accesibility Console functionality.
* Integration with the ConsolesManager is managed by AbstractConsole.
* 
* @see		com.fwamarco.consoles.AbstractConsole
* @see 		com.fwamarco.consoles.ConsolesManager
* 
* @author	Federico Campo Piombi
* @version	1.0
* @date 	27 Nov 2006
*/
class com.fwamarco.consoles.Accesibilidad extends AbstractConsole {
	
	//layout elements
	private var bt_close:Button;
	private var chk_tooltips:MovieClip;
	private var txt_tooltips_fontsize:TextField;
	private var bt_fontsize_up:Button;
	private var bt_fontsize_down:Button;
	private var chk_barrido:MovieClip;
	private var bt_interval_up:Button;
	private var bt_interval_down:Button;
	private var txt_barrido_interval:TextField;
	private var t_title_activateBarrido:TextField;
	private var t_title_intervalTimeout:TextField;
	private var t_title_activateTooltip:TextField;
	private var t_title_fontSize:TextField;
	
	//properties
	private var tooltip_fontsize:Number;
	private var barrido_interval:Number;
	private var config:Object;
	
	/**
	* Constructor
	*/
	public function Accesibilidad(){
		super()
		config = FWAConfiguration.main.accesibility
		init()
		initTextFields();
	}
	
	/**
	* Initialises all the layout elements and applies default configuration.
	* @return	Void
	*/
	public function init():Void {
		bt_close.onRelease = Delegate.create(this, hide)
		bt_fontsize_up.onRelease  = Delegate.create(this, setTooltipFontsizeUp)
		bt_fontsize_down.onRelease=Delegate.create(this, setTooltipFontsizeDown)
		bt_interval_up.onRelease = Delegate.create(this, setBarridoTimeoutUp)
		bt_interval_down.onRelease = Delegate.create(this, setBarridoTimeoutDown)
		
		chk_tooltips.onChange = Delegate.create(this, setTooltipsEnabled)
		chk_barrido.onChange = Delegate.create(this, setBarridoEnabled)
		
		tooltip_fontsize = Number(config.tooltips.font.size._value)
		barrido_interval = Number(config.barrido.speed._value)
		updateBarrido()
		
		var tooltip_enabled:Boolean = (config.tooltips.enabled._value.toLowerCase()=="true")? true : false
		txt_tooltips_fontsize.text = String(tooltip_fontsize)
		chk_tooltips.setValue(tooltip_enabled)
	}
	
	/**
	* Displays the console and intialises the display configuration values
	* @return	Void
	*/
	public function show():Void {
		super.show()
	}
	
	
	
	/**
	* Initialises the consoles TextFields content from the main language file
	* Updates the display properties of the related elements
	*/
	private function initTextFields():Void {
		//get TextFormats
		var tf_title_activateBarrido:TextFormat = t_title_activateBarrido.getTextFormat();
		var tf_title_intervalTimeout:TextFormat = t_title_intervalTimeout.getTextFormat();
		var tf_title_activateTooltip:TextFormat = t_title_activateTooltip.getTextFormat();
		var tf_title_fontSize:TextFormat = t_title_fontSize.getTextFormat();
		
		//apply labels
		t_title_activateBarrido.text = Language.getLabel("conf_activar_barrido")
		t_title_intervalTimeout.text = Language.getLabel("conf_interval_timeout")
		t_title_activateTooltip.text = Language.getLabel("conf_activar_textos")
		t_title_fontSize.text = Language.getLabel("conf_fontsize")
		
		
		//apply TextFormats
		t_title_activateBarrido.setNewTextFormat(tf_title_activateBarrido);
		t_title_intervalTimeout.setNewTextFormat(tf_title_intervalTimeout);
		t_title_activateTooltip.setNewTextFormat(tf_title_activateTooltip);
		t_title_fontSize.setNewTextFormat(tf_title_fontSize);
	}
	
	
	//TOOLTIP METHODS
	
	/**
	* Increases the font size 1px up
	* @return	Void
	*/
	private function setTooltipFontsizeUp():Void {
		tooltip_fontsize ++
		txt_tooltips_fontsize.text = String(tooltip_fontsize)
		config.tooltips.font.size._value = String(tooltip_fontsize)
	}
	
	/**
	* Decreases the font size 1px down
	* @return	Void
	*/
	private function setTooltipFontsizeDown():Void {
		tooltip_fontsize --
		txt_tooltips_fontsize.text = String(tooltip_fontsize)
		config.tooltips.font.size._value = String(tooltip_fontsize)
	}
	
	/**
	* Enables/disables the tooltip box
	* @param	val (Boolean) true if the subtitles should be displayed
	* @return	Void
	*/
	private function setTooltipsEnabled(val:Boolean):Void {
		config.tooltips.enabled._value = String(val)
	}
	
	
	//BARRIDO METHODS
	/**
	* Enables/disables the Barrido execution
	* If the Barrido is enabled, then it also executes the Barrido,
	* if not, it clears all the Barrido elements call.
	* @param	val (Boolean) true if the Barrido should executed
	* @return	Void
	*/
	private function setBarridoEnabled(val:Boolean):Void {
		Barrido.enabled = val
		if(val == true){
			Barrido.execute()
		}else{
			Barrido.clear()
		}
	}
	
	/**
	* Increases the Barrido Timeout
	* @return	Void
	*/
	private function setBarridoTimeoutUp():Void {
		barrido_interval ++
		updateBarrido()
	}
	
	/**
	* Decreases the Barrido Timeout
	* @return	Void
	*/
	private function setBarridoTimeoutDown():Void {
		if(barrido_interval>2){
			barrido_interval --
		}
		updateBarrido()
	}
	
	/**
	* Updates the Barrido Timeout value, and the console display number
	* @return	Void
	*/
	private function updateBarrido():Void {
		Barrido.timeout = barrido_interval
		txt_barrido_interval.text = String(barrido_interval)
	}
}