import com.fwa.system.FWAConfiguration;
import com.fwamarco.consoles.AbstractConsole;
import com.fwamarco.consoles.ConsolesManager;
import mx.utils.Delegate;

/**
* This class manages the Help Console functionality.
* Integration with the ConsolesManager is managed by AbstractConsole.
* 
* @see		com.fwamarco.consoles.AbstractConsole
* @see 		com.fwamarco.consoles.ConsolesManager
* 
* @version	1.0
* @since 	17 Jun 2007
*/
/**
* @ignore
* MODIFICATION HISTORY
* @ignore
*/
class com.fwamarco.consoles.Help extends AbstractConsole {
	//layout elements
	private var clip_bg:MovieClip;
	private var bt_close:Button;
	private var bt_scrollUp:Button;
	private var bt_scrollDown:Button;
	private var t_help:TextField;
	
	
	/**
	* Constructor
	*/
	public function Help(){
		bt_close.onRelease = Delegate.create(ConsolesManager, ConsolesManager.closeAll);
		
		bt_scrollUp.onPress = Delegate.create(this, beginScrollUp);
		bt_scrollDown.onPress = Delegate.create(this, beginScrollDown);
		bt_scrollUp.onRelease = bt_scrollUp.onReleaseOutside = Delegate.create(this, endScroll);
		bt_scrollDown.onRelease = bt_scrollDown.onReleaseOutside = Delegate.create(this, endScroll);
	}
	
	/**
	* Displays the Help Console.
	* Fills the help textfield with the Help content
	* @return 	Void
	*/
	public function show():Void {
		super.show();
		displayHelp();
		updateScrolls();
	}
	
	/**
	* Fills the help textfield with the Help data
	*/
	private function displayHelp():Void {
		//get TextFormats
		var tf_help:TextFormat = t_help.getTextFormat();
		
		//apply labels
		t_help.text = FWAConfiguration.help;
		
		//apply TextFormats
		t_help.setNewTextFormat(tf_help);
	}
	

	/**
	* Starts a loop to scroll up
	*/
	private function beginScrollUp():Void {
		this.onEnterFrame = Delegate.create(this, scrollUp);
	}
	
	/**
	* Starts a loop to scroll down
	*/
	private function beginScrollDown():Void {
		this.onEnterFrame = Delegate.create(this, scrollDown);
	}
	
	/**
	* Finishes scrolling the TextField
	*/
	private function endScroll():Void {
		delete this.onEnterFrame;
	}
	
	/**
	* Scrolls the TextField up
	*/
	private function scrollUp():Void {
		t_help.scroll ++
	}
	
	/**
	* Scrolls the TextField down
	*/
	private function scrollDown():Void {
		t_help.scroll --
	}
	
	/**
	* Updates the scroll buttons visibility according to the
	* Log's TextField scroll properties
	*/
	private function updateScrolls():Void {
		//trace("max:"+t_log.maxscroll+" bott:"+t_log.bottomScroll+" scroll:"+t_log.scroll);
		var hasScroll:Boolean = (t_help.maxscroll >1);
		bt_scrollUp._visible = hasScroll;
		bt_scrollDown._visible = hasScroll;
	}
}