import com.wunderman.debug.Tracer;
import com.wunderman.ui.IPreloaderListener;
import com.wunderman.ui.Window;
import mx.utils.Delegate;
/**
* @class 	PreloaderListener
* @tooltip 	Creates an object to use as Listener when preloading content (Windows).
* @author	Federico Campo Piombi
* @version	1.0
* @date 	06 Nov 2006
* 
* @see 		com.wunderman.ui.Window
* @see 		com.wunderman.ui.Windows
*/
class com.parker.content.PreloaderListener implements IPreloaderListener {
	//properties
	private var loader_window:Window;
	
		
	/**
	* @constructor
	*/
	public function PreloaderListener(){
		//create the window to use as preloader display
		//var loader_depth:Number = _root.getNextHighestDepth()+100
		//loader_window = new Window(_root, "loader"+loader_depth, loader_depth, true)
		//trace("loader created: loader"+loader_depth)
	}
	
	
	//public methods
	/**
	* @method 	getListener
	* @tooltip 	Returns an object with the Listener events
	* @return	Object
	*/
	public function getListener():Object {
		//create the window to use as preloader display
		var loader_depth:Number = _root.clip_main.__FOREGROUND.getNextHighestDepth()
		loader_window = new Window(_root.clip_main.__FOREGROUND, "loader"+loader_depth, loader_depth, true)
		trace("loader created: loader"+loader_depth)
		
		//create the preloader listener
		var loader_listener = new Object()
		loader_listener.onLoadStart = Delegate.create(this, onLoadStart)
		loader_listener.onLoadProgress = Delegate.create(this, onLoadProgress)
		loader_listener.onLoadComplete = Delegate.create(this, onLoadComplete)
		loader_listener.onLoadInit = Delegate.create(this, onLoadInit)
		loader_listener.onLoadError= Delegate.create(this, onLoadError)
		
		return(loader_listener)
	}
	
	
	//events
	/**
	* @event 	onLoadStart
	* @tooltip 	Invoked when a call to MovieClipLoader.loadClip() has begun to download a file.
	* @param	tmc (MovieClip) Movie being loaded
	*/
	private function onLoadStart(tmc:MovieClip):Void {
		loader_window.attachContent("clip_loader")
		loader_window.getContentClip()._x = Stage.width/2;
		loader_window.getContentClip()._y = Stage.height/2;
		loader_window.show()
		tmc.stop()
		tmc._alpha = 0
	}
	
	/**
	* @event 	onLoadProgress
	* @tooltip 	Invoked every time the loading content is written to the hard disk during the loading process.
	* @param	tmc (MovieClip) Movie being loaded
	* @param	bL 	(Number) Amount of bytes loaded
	* @param	bT 	(Number) Total bytes to load
	*/
	private function onLoadProgress(tmc:MovieClip, bL:Number, bT:Number):Void {
		loader_window.show()
		loader_window.getContentClip().setProgress(bL, bT)
	}
	
	/**
	* @event 	onLoadComplete
	* @tooltip 	Invoked when a file that was loaded with MovieClipLoader.loadClip() is completely downloaded.
	* @param	tmc (MovieClip) Movie loaded
	*/
	private function onLoadComplete(tmc:MovieClip):Void {
	}
	
	/**
	* @event 	onLoadInit
	* @tooltip 	Invoked when the actions on the first frame of the loaded clip have been executed.
	* @param	tmc (MovieClip) Movie loaded
	*/
	private function onLoadInit(tmc:MovieClip):Void {
		tmc.alphaTo(100, 0.5, "easeOutQuint")
		tmc.play()
		loader_window.close()
	}
	
	/**
	* @event 	onLoadComplete
	* @tooltip 	Invoked when a file loaded with MovieClipLoader.loadClip() has failed to load.
	* @param	tmc (MovieClip) Movie loaded
	* @param	errcode (String)  A string that explains the reason for the failure, either "URLNotFound" or "LoadNeverCompleted"
	* @param	httpstatus (Number) The HTTP status code returned by the server
	*/
	private function onLoadError(tmc:MovieClip, errcode:String, httpstatus:Number):Void {
		Tracer.output("error loading:"+errcode, "onLoadError", "PreloaderListener")
		tmc._window.close()
		loader_window.close()
	}
}