import com.parker.core.URLs;
import com.parker.core.Configuration;
import com.parker.content.WindowsProxy;
import com.parker.content.PreloaderListener;
import com.wunderman.ui.Windows;
import com.wunderman.ui.Placeholders;
import com.wunderman.ui.Window;
import mx.utils.Delegate;

/**
* @class	Main
* @tooltip 	This class has the initialisation code for the site's content
* @author	Federico Campo Piombi
* @version	1.0
* @date 	01 Nov 2006
*/
class com.parker.ui.Main extends MovieClip {
	
	//properties
	private var wproxy:WindowsProxy;
	private var windows:Windows;
	private var botonera:Window;
	
	/**
	* @constructor
	*/
	public function Main(){
		//Create main Placholders
		Placeholders.removeIfExists(true)
		Placeholders.createPlaceholder("__BACKGROUND", this, -1)
		Placeholders.createPlaceholder("__CONTENT",this, 1)
		Placeholders.createPlaceholder("__STATIC", this, 2)
		Placeholders.createPlaceholder("__POPUPS", this, 3)
		Placeholders.createPlaceholder("__ALERTS", this, 4)
		Placeholders.createPlaceholder("__FOREGROUND", this, 5)
	}
	
	
	/**
	* @method 	init
	* @tooltip 	Initialises the main components of the site
	* @return	Void
	*/
	public function init():Void {
		//Initialise vars
		windows = Windows.getInstance()
		wproxy = WindowsProxy.getInstance()
		
		//init content
		initStaticContent()
		initPrehome()
	}
	
	
	/**
	* @method 	openContent
	* @tooltip 	Opens the content related to a Menu Item
	* @param	config (Objec) Selected item's configuration object
	* @return	Void
	*/
	private function openContent(config:Object):Void {
		if(config.file!==undefined){
			//open filename
			var wnd:Window = wproxy.openWindow(config.file._value)
			var bg:Window = wproxy.loadBackground(URLs.backgrounds+config.background._value)
			var wnd_preloader:PreloaderListener = new PreloaderListener()
			var bg_preloader:PreloaderListener = new PreloaderListener()
			wnd.addListener(wnd_preloader.getListener())
			bg.addListener(bg_preloader.getListener())
			
		}else if(config.url!==undefined){
			var target:String = (config.url_target!==undefined)? config.url_target._value : "_blank"
			getURL(config.url._value, target)
		}
	}
	
	
	/**
	* @method 	initStaticContent
	* @tooltip 	Attaches and inits the Logo and Buttons Nav
	* @return 	Void
	*/
	private function initStaticContent():Void {
		var logo:Window = wproxy.attachContent("clip_logo", "__FOREGROUND")
		logo._x = logo._y = 20
		
		botonera = wproxy.attachContent("clip_botonera", "__STATIC")
		botonera._x = 0
		botonera._y = 394
		botonera.hide()
		
		var clip_botonera:MovieClip = botonera.getContentClip()
		clip_botonera.createMenu(Configuration.getStructure())
		clip_botonera.onMenuItemSelected = Delegate.create(this, openContent)
	}
	
	
	/**
	* @method 	inintPrehome
	* @tooltip 	Loads the prehome clip.
	* @return 	Void
	*/
	private function initPrehome():Void {
		if(Configuration.getConfig().country_selection.enabled._value == "true"){
			var prehome:Window = wproxy.openWindow("pre_home.swf")
			var prehome_close:Object = new Object()
			var prehome_load:PreloaderListener = new PreloaderListener()
			
			prehome_close.onClose = Delegate.create(this, initHome)
			prehome.addListener(prehome_close)
			prehome.addListener(prehome_load.getListener())
			
		}else{
			initHome()
		}
	}
	
	/**
	* @method	initHome
	* @tooltip 	Loads and initialises the Site's home
	* @return	Void
	*/
	private function initHome():Void {
		//var home:Window = wproxy.openWindow("home.swf")
		openContent(Configuration.getStructure().section[0])
		
		//display botonera
		botonera._y = 500
		botonera.show()
		
		var bt_clip:MovieClip = botonera.getContainerClip()
		bt_clip._alpha = 0
		bt_clip.tween(["_alpha","_y"], [100,394], 1, "easeOutQuint",0)
		
	}
}