import mx.utils.Delegate

/**
* @class 	MessageBox
* @tooltip 	This class creates a box to display messages (alerts, info, etc).
* 			Arranges buttons, textfields, etc.
* @author	Federico Campo Piombi
* @version	1.0
* @date 	
*/
class com.parker.ui.MessageBox extends MovieClip {
	
	//properties
	private var msg_cb:Object
	
	//layout elements
	private var t_titulo:TextField
	private var t_message:TextField
	private var bt_volver:Button
	
	/**
	* @constructor
	*/
	public function MessageBox(){
		bt_volver.onRelease = Delegate.create(this, onClose)
	}
	
	
	/**
	* @method 	setAlert
	* @tooltip 	Arranges the content of the alert Clip to display the title, message and distribution of elelments
	* 			according to the <type> param. Also stores the callback information in <msg_cb>
	* @param	title (String) Title of the message
	* @param	msg (String) Content of the message
	* @param	type (String) Type of the message
	* @param	callback (Object) Object with callbacks to execute when pressing the different buttons
	*/
	public function setAlert(title:String, msg:String, type:String, callback:Object):Void {
		msg_cb = callback
		t_titulo.text = title
		t_message.text = msg
		setAlertType(type)
	}
	
	/**
	* @method 	setAlertType
	* @tooltip 	Arranges the layout elements accordign to the provided alert type
	* @param	type (String) Indicates the type of alert to be created
	* @return 	Void
	*/
	private function setAlertType(type:String):Void {
		switch(type.toLowerCase()) {
			case "alert":
				bt_volver._visible = true
				break;
			case "none":
				bt_volver._visible = false
				break;
		}
	}
	
	/**
	* @method 	closeAlert
	* @tooltip 	Closes the current Messagebox
	* @return	Void
	*/
	private function closeAlert():Void {
		_parent.close()
	}
	
	
	//events
	/**
	* @event 	onClose
	* @tooltip 	Event executed when the button Volver is pressed
	* @return	Void
	*/
	private function onClose():Void {
		var scope = (msg_cb.onClose_scope==undefined)? null : msg_cb.onClose_scope
		var args:Array = (msg_cb.onClose_args==undefined)? [] : msg_cb.onClose_args
		msg_cb.onClose.apply(scope, args)
		closeAlert()
	}
}