﻿
import com.wunderman.core.ListenerBroadcaster;

class com.wunderman.controllers.repeaters.ItemList {
	
	//VARIABLES////////////////////////////////////////////////////////////////////
	//init vars
	var Broadcaster:ListenerBroadcaster
	
	//config vars
	var tCols:Number
	var tRows:Number
	var hSpace:Number
	var vSpace:Number
	var thumbW:Number
	var thumbH:Number
	var maxDisplayThumbs:Number
	
	//internal vars
	var tThumbs:Number
	var thumbsDisplayed:Number
	var pagesCount:Number
	var pageCurrent:Number
	var dFrom:Number
	var dTo:Number
	
	
	//CONSTRUCTOR//////////////////////////////////////////////////////////////////
	public function ItemList() {
		this.Broadcaster = new ListenerBroadcaster()
		this.Broadcaster.broadcastMessage("onListLoaded")
	}

	
	//CORE METHODS//////////////////////////////////////////////////////////////////
	public function createList(thumbsCount:Number, oConfig:Object){
		//apply current config
		this.applyConfig(oConfig)
		this.tThumbs = thumbsCount
		
		//calculate thumbs amount and paginate
		if(this.tThumbs>this.maxDisplayThumbs){
			this.pagesCount = Math.ceil(this.tThumbs/this.maxDisplayThumbs)
		}else{
			this.pagesCount = 1
		}
		
		//display Thumbs
		this.Broadcaster.broadcastMessage("onListCreated",[this.pagesCount])
	}
	
	public function displayPage(default_page:Number){
		//set current page
		if(default_page>this.pagesCount){
			this.pageCurrent = this.pagesCount
		}else if(default_page<1){
			this.pageCurrent = 1
		}else{
			this.pageCurrent = default_page
		}
		var thumbsToDisplay:Number
		
		//calculate pictures per page
		if(this.pageCurrent == this.pagesCount){
			thumbsToDisplay = this.tThumbs-((this.pagesCount-1)*this.maxDisplayThumbs)
		}else{
			thumbsToDisplay = this.maxDisplayThumbs
		}
		
		//clear prev thumbs
		this.clearAllItems()
		
		//start creating thumbnails
		var tCounter:Number = 0
		var tRows:Number = 0
		var from:Number = (this.maxDisplayThumbs*(this.pageCurrent-1))
		var to:Number = (this.tThumbs>this.maxDisplayThumbs)? from+(thumbsToDisplay) : this.tThumbs
		//trace("PAGES: "+ pageCurrent+"/"+pagesCount)
		//trace("ITEMS: "+ from+"/"+to)
		
		//Start Loop
		for(var i=from; i<to; i++){
			
			//calculate item location
			var thumbX = (this.thumbW+this.hSpace) * tCounter
			var thumbY = (this.thumbH+this.vSpace) * tRows
			
			//broadcast thumb creation message
			this.Broadcaster.broadcastMessage("onItemCreated",[i, thumbX, thumbY])
			
			//update rows info
			if(tCounter<(tCols-1)){
				tCounter++ 
			}else{
				tCounter = 0
				tRows ++
			}
		}
		this.thumbsDisplayed = thumbsToDisplay
		this.dFrom = from
		this.dTo = to
		
		//broadcast end message
		this.Broadcaster.broadcastMessage("onPageCreated",[this.pageCurrent,this.pagesCount])
	}
	
	public function clearAllItems(){
		for(var i=this.dFrom; i<this.dTo; i++){
			//broadcast deletion message
			this.Broadcaster.broadcastMessage("onItemDeleted",[i])
		}
	}
	
	private function applyConfig(cObj:Object){
		this.maxDisplayThumbs = (cObj.items_maxdisplay==undefined)? cObj.columns*cObj.rows : cObj.items_maxdisplay
		this.tCols = cObj.columns
		this.tRows = cObj.rows
		this.hSpace = cObj.h_space
		this.vSpace = cObj.v_space
		this.thumbW = cObj.item_w
		this.thumbH = cObj.item_h
	}
	
	//GET Methods///////////////////////////////////////////////////////////////////////////
	public function getPages():Object {
		return( {current:this.pageCurrent,total:this.pagesCount} )
	}
	public function getItemPage(itemIndex:Number):Number {
		return( Math.ceil((itemIndex+1)/this.maxDisplayThumbs) )
	}
	
	
	//LISTENER BROADCASTER INTEGRATION//////////////////////////////////////////////////////
	public function addListener(objListener:Object){
		return( this.Broadcaster.addListener(objListener) )
	}
	public function removeListener(objListener:Object){
		return (this.Broadcaster.removeListener(objListener))
	}

}