/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.iterator.*;

/**
   
  @tiptext Vector is an ordered collection of items.
*/
class asdk.Vector extends asdk.AbstractCollection
{
	private var items:Array;
	
	function Vector( items )
	{
		if( items == undefined )
		  this.items = new Array();
		else
		  this.items = items;
	}
	
	function New() : Collection
	{
		return new Vector();
	}	
	
	function addAllItems( collection : Collection ) : Boolean
	{
		return this.addAll( collection );
	}
	
	function insertItemAt( index : Number, element )
	{
		if( index >= getSize() )
		  throw new Error( "index out of bounds" );
		
		this.items.splice( index, 0, element );
	}
	
	function insertItemsAt( index : Number, collection : Collection )
	{
		if( index >= items.length || index < 0 )
		  throw new Error( "index out of bounds" );
		  
	    var iter : Iterator = collection.iterator();
		
        for( var i = index; iter.hasNext(); i++ )
          insertItemAt( i, iter.getNext() );
	}
	
	function getItemAt( index : Number )
	{
		return items[ index ];
	}
	
	function getFirstItem() 
	{
		return getItemAt( 0 );
	}
	
	function getLastItem()
	{
		return getItemAt( getSize() - 1 );
	}
	
	function clone() : Vector
	{
		return new Vector( items.slice() );
	}
	
	function getFirstIndexOf( element, index : Number ) : Number
	{
		if( index == undefined )
		  index = 0;
		  
		for( var i = index; i < getSize(); i++ )
		  if( getItemAt( i ) == element )
		    return i;
	   
	   return -1;
	}
	
	function getLastIndexOf( element, index : Number ) : Number
	{
		if( index == undefined )
		  index = 0;
		  
		for( var i = getSize() - 1; i >= index; i-- )
		  if( getItemAt( i ) == element )
		    return i;
		
		return -1;		
	}
	
	function removeItemAt( index : Number ) : Boolean
	{
		if( index >= getSize() || index < 0 )
		{
		  throw new Error( "index out of bounds" );
		  return false;
		}
		
		items.splice( index, 1 );
		return true;
	}
	
	function getRange( startIndex : Number, endIndex : Number ) : Collection
	{
		if( startIndex < 0 || startIndex >= getSize() || endIndex < 0 || endIndex >= getSize() )
		  throw new Error( "index out of bounds" );
		
		var collection : Vector = new Vector();
		
		for( var i = startIndex; i <= endIndex; i++ )
		  collection.addItem( getItemAt( i ) );
		
		return collection;		
	}
	
	function removeRange( startIndex : Number, endIndex : Number ) : Collection
	{
		var collection : Collection = getRange( startIndex, endIndex );
		removeAllItems( collection ); // this needs to be optimized
		return collection;
	}
	
	function replaceItem( element, index : Number )
	{
		var oldValue = getItemAt( index );
		items[ index ] = element;
		return oldValue;
	}
	
	
	
	/****************  asdk.Collection methods ***************************/
	
	function addItem( item ) : Boolean
	{
		this.items.push( item );
		return true;
	}	
	
	function equals( object, comparator : IComparator ) : Boolean
	{
		if( !(object instanceof Vector) )
		  return false;
		
		if( object.getSize() != this.getSize() )
		  return false;
		  
		var length : Number = this.getSize();
		
		for( var i = 0; i < length; i++ )
		{
			var obj = object.getElementAt( i );
			
			if( comparator != undefined )
			{
				if( !comparator.equal( obj, items[ i ] ) )
					return false;
			}
			else if( obj != items[ i ] )
			{
				return false;
			}
		}
			
	    return true;	
	}	
	
	function iterator() : Iterator
	{
		return new VectorIterator( this );		
	}	
	
	function getSize() : Number
	{
		return items.length;
	}
	
}