/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.xml.*;
import mx.utils.Delegate;

class asdk.xml.Document extends Parent
{
	private var xmlDoc:XML;
	private var loaded;
	private var onLoadCallback:Function;
	
	public function Document( xmlDocObj )
	{
		loaded = false;
		setOwnerDocument( this );
		
        XML.prototype.ignoreWhite = true;
        
		if( xmlDocObj != undefined )
			parse( xmlDocObj );
	}
	
	public function parse( xmlDocObj )
	{
		if( typeof( xmlDocObj ) == "string" )
        {
			this.xmlDoc = new XML( xmlDocObj );
            this.xmlDoc.ignoreWhite = true;
        }
		else
			this.xmlDoc = xmlDocObj;
		
		if( this.xmlDoc.hasChildNodes() )
		{
            var rootElement = NodeFactory.createNode( this.xmlDoc.childNodes[ 0 ], this );
            setRootElement( rootElement );
		}
		
		loaded = true;
	}
	
	public static function load( url:String, callback:Function ) : Document
	{
		var docObj = new Document();
		var xmlObj = new XML();
		xmlObj.onLoad = Delegate.create( docObj, docObj.onLoad );
		xmlObj.onData = Delegate.create( docObj, docObj.onData );
		docObj.setOnLoadCallback( callback );
		xmlObj.load( url );
		return docObj;
	}
	
	public function setOnLoadCallback( callback:Function )
	{
		this.onLoadCallback = callback;
	}
	
	public function onLoad( success )
	{
		loaded = success;
		onLoadCallback.call( null, this );
	}
	
	public function onData( xmlSource:String )
	{
		parse( xmlSource );
		onLoad( true );
	}
	
	public function isLoaded()
	{
		return loaded;
	}
	
	public function getRoot():Element
	{
        for( var node = children.firstNode; node != undefined; node = node.nextNode )
            if( node instanceof Element )
                return node;
                
		return null;
	}
	
	public function newRoot():Element
	{
		var element = new Element();
		setRootElement( element );
        return element;
	}
	
	public function setRootElement( newRoot:Element ):Element
	{
	    var root = getRoot();
        
    	if( root != null )
      	  root.replaceWith( newRoot );
	    else
    	  addChild( newRoot );
	
    	return newRoot;
    }
	
  	public function setRoot( name:String, prefix:String, namespace:String ):Element
    {
	    var root = newRoot();
		
		if( namespace != undefined )
		  root.setNamespace( prefix, namespace ); 
		else if( prefix != undefined )
		  root.setName( name, prefix );
		else
		  root.setName( name );
			
    	return root;
    }	
    
    public function toString()
    {
        return xmlDoc.toString();
    }
}