/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.xml.*;

class asdk.xml.Parent extends asdk.xml.Child
{
	public var children:NodeList;
	
	public function Parent()
	{
		this.children = new NodeList();
	}	

   /**
    * Add the specified child to my list of children and return the child.
    * @param child The child.
    */    
  	public function addChild( child:Child ) : Child
  	{
  		child.setParent( this );
        child.setOwnerDocument( this.document );
    	children.append( child );
    	return child;
  	}
	
   /**
    * Replace the old child with the new child.
    * @param oldChild The old child.
    * @param newChild The new child.
    */    
    public function replaceChild( oldChild:Child, newChild:Child ) : Void
    {
	    this.children.replace( oldChild, newChild );
	    newChild.setParent( this );
        newChild.setOwnerDocument( this.document );
    }	    
    
   /**
    * Return true if I have one or more children.
    */    
    public function hasChildren() : Boolean
    {
        return !children.isEmpty();
    }

   /**
    * Return true if I have one or more child Elements.
    */
    public function hasElements() : Boolean
    {
        return getElements().hasNext();
    }
    
   /**
    * Return true if I have an Element with the specified name.
    * @param name The name.
    */    
    public function hasElement( name:String ) : Boolean
    {
        return (getElement( name ) != undefined);    
    }
    
   /**
    * If no arguments passed, return an enumeration of my child Elements.
    * If only name is passed, return an enumeration over all my Elements with the specified name.
    * If both name and namespace are passed, return an enumeration over all my Elements 
    * with the specified namespace and name.
    */    
    public function getElements( name:String, namespace:String ) 
    {
        if( name == undefined && namespace == undefined )
            return new ElementsFilter( children );
        else if( name != undefined && namespace == undefined )
        {
            var selected : NodeList = new NodeList();

            for( var node = children.firstNode; node != undefined; node = node.nextNode )
              if( node instanceof Element && node.name == name )
                selected.append( new Selection( node ) );

            return new Elements( selected );        
        }
        else if( name != undefined && namespace != undefined )
        {
            var selected : NodeList = new NodeList();

            for( var node = children.firstNode; node != undefined; node = node.nextNode )
              if( node instanceof Element && node.hasName( namespace, name ) )
                selected.append( new Selection( node ) );

            return new Elements( selected );        
        }
        else
            throw new Error( "invalid argument" );       
    }
    
    /**
     * Return my first Element node with the specified name or namespace and name,
     * or null if I have none.
     * @param namespace The namespace.
     * @param name The name.
     */    
    public function getElement( name:String, namespace:String )
    {
        if( name != undefined && namespace == undefined )
        {
            for( var node = children.firstNode; node != null; node = node.nextNode )
				if( (node instanceof Element) && node.name == name )
                    return node;

            return undefined;        
        }
        else if( name != undefined && namespace != undefined )
        {
            for( var node = children.firstNode; node != null; node = node.nextNode )
              if( node instanceof Element && node.hasName( namespace, name ) )
                return node;

            return null;        
        }
        else
            throw new Error( "either name or both name and namespace are required" );                   
    }
    
   /**
    * Return the Element node at the specified index, using 1 as the base
    * index for Xpath compatibility.
    * @throws Error If the index was illegal.
    */    
    public function getElementAt( index ) 
    {
        var n = 0;

        for( var node = children.firstNode; node != undefined; node = node.nextNode )
          if( node instanceof Element && ++n == index )
            return node;

        throw new Error( index + " is an invalid index" );    
        return undefined;
    }    
    
    public function getXPathElement( xpathExpression )
    {
        throw new Error( "XPath expressions are not supported yet" );    
        return undefined;
    }
    
   /**
    * Return the first Element, or null if there is none.
    */    
    public function getFirstElement()
    {
        for( var node = children.firstNode; node != null; node = node.nextNode )
          if( node instanceof Element )
            return node;

        return undefined;    
    }
    
   /**
    * Set the element node at the specified index, using 1 as the base
    * index for Xpath compatibility, and return the previous value.
    * @param index The index.
    * @param element The element.
    * @throws Error If the index was illegal.
    */    
    public function setElementAt( index, element ) 
    {
        var previous = getElementAt( index );
        previous.replaceWith( element );
        return previous;    
    }
    
   /**
    * Remove and return the Element node at the specified index, using 1 as the base
    * index for Xpath compatibility.
    * @param index The index.
    * @throws Error If the index was illegal.
    */    
    public function removeElementAt( index ) 
    {
        var element = getElementAt( index );
        element.remove();
        return element;    
    }
    
   /**
    * When no arguments passed, add a new element with no name.
    * If the nameOrElement argument is an instance of Element, add the specified element,
    * otherwise add a new Element with the specified name and optionally namespace prefix name.
    * @return The new element.
    */
    public function addElement( nameOrElement, prefix:String )
    {
        // name could be either name or Element or NOTHING
        // prefix cam be present only with name, but it is optional
        
        if( nameOrElement == undefined )
        {
            var element = new Element();
            element.setOwnerDocument( this.document );
            return addElement( element );
        }
        else if( nameOrElement instanceof Element )
        {
            addChild( element );
            return element;
        }
        else
        {
            if( prefix == undefined )
                return addElement().setName( nameOrElement );
            else
                return addElement().setName( prefix, nameOrElement );
        }
    }

   /**
    * When no arguments passed, insert a new element with no name.
    * If the nameOrElement argument is an instance of Element, insert the specified element,
    * otherwise insert a new Element with the specified name and optionally namespace prefix name.
    * @return The new element.
    */    
    public function insertElement( nameOrElement, prefix:String )
    {
        if( nameOrElement == undefined )
        {
            var element = new Element();
            element.setOwnerDocument( this.document );
            return insertElement( element );        
        }
        else if( nameOrElement instanceof Element )
        {
            insertElement( element );
            return element;        
        }
        else
        {
            if( prefix == undefined )
                return insertElement().setName( nameOrElement );
            else
                return insertElement().setName( prefix, nameOrElement );
        }
    }
    
   /**
    * If name and namespace are not passed, remove all the elements.
    * Otherwise, remove elements with the name and optionally namespace.
    */    
    public function removeElements( name:String, namespace:String )
    {
        // name is optional
        if( name == undefined )
            getElements().remove();
        else
        {
            var elements : Elements;
            
            if( namespace != undefined )
                elements = getElements( name, namespace );
            else
                elements = getElements( name );            
                
            elements.remove();
            elements.reset();
        }       
    }
    
   /**
    * Remove and return the first element with the specified name, or return
    * null if none was found.
    * @param name The name.
    * @param name The namespace. This parameter is optional.
    */    
    public function removeElement( name:String, namespace:String ) 
    {
        // namespace is optional    
        var element;
        
        if( namespace == undefined )
            element = getElement( name );
        else
            element = getElement( name, namespace );

        if( element != null )
          element.remove();

        return element;        
    }
    
    public function removeXPathElement( xpathExpression )
    {
        throw new Error( "XPath expressions are not supported yet" );        
        return undefined;
    }
    
    /**
     * Return the value of the namespace with the specified prefix, or null if
     * there is none. This operation searches up the element hierarchy starting
     * at this element.
     * @param prefix The prefix.
     */
    public function getNamespace( prefix : String ) : String
    {
        return undefined;
    }    
}