import com.fwa.events.ListenerBroadcaster;
import com.fwa.events.IListenable;
import mx.utils.Delegate;

/**
* Volume controller Class
* Provides integration with a volume control component giving drag capabilities and volume update message broadcasting
* @version	1.5
* @date 	10 Oct 2006
*/
class com.fwa.controllers.media.Volume extends MovieClip implements IListenable {
	
	//properties
	private var Broadcaster:ListenerBroadcaster
	private var volume_int:Number
	private var clip_drag_bg:MovieClip;
	private var clip_drag:MovieClip;
	
	
	/**
	* Constructor
	*/
	public function Volume(){
		Broadcaster = new ListenerBroadcaster()
		clip_drag.onPress = Delegate.create(this, volumePress)
		clip_drag.onRelease = clip_drag.onReleaseOutside = Delegate.create(this, volumeRelease)
		setEnabled(true)
	}

	
//PUBLIC METHODS
	/**
	* Retrieves the current volume value
	* @return 	Number
	*/
	public function getVolume():Number {
		return volume_int
	}
	
	/**
	* Updates the bar position according to the new volume value and updates.
	* @param	vol (Number) the new volume value
	* @return	Void
	*/
	public function setVolume(vol:Number):Void {
		vol = (vol<0)? 0 : vol
		vol = (vol>100)? 100 : vol
		volume_int = vol
		clip_drag._x = Math.ceil(vol * (clip_drag_bg._x+clip_drag_bg._width-clip_drag._width) / 100)
		onVolumeUpdated()
	}
	
	/**
	* Enables or disables the control
	* @param	val (Boolean) true if enabled, otherwise false
	* @return	Void
	*/
	public function setEnabled(val:Boolean):Void {
		clip_drag.enabled = val
		clip_drag.useHandCursor = val
	}
	
	/**
	* Retrieves true if the control is enabled
	* @return	Boolean
	*/
	public function getEnabled():Boolean {
		return(clip_drag.enabled)
	}
	
	
//PRIVATE METHODS
	/**
	* Starts the volume update loop (when the bar is pressed and dragged)
	* @return	Void
	*/
	private function volumePress():Void {
		// volume scrubbing
		var min_vol:Number = clip_drag_bg._x
		var max_vol:Number = clip_drag_bg._x + clip_drag_bg._width - clip_drag._width
		clip_drag.startDrag(false, min_vol, clip_drag._y, max_vol, clip_drag._y)
	}
	
	/**
	* Stops the volume update loop (when the bar is released)
	* @return	Void
	*/
	private function volumeRelease():Void {
		stopDrag()
		adjustVolume()
	}
	
	/**
	* Calculates the volume value according to the bar position
	* @return	Void
	*/
	private function adjustVolume():Void {
		volume_int = Math.round(clip_drag._x *100 /(clip_drag_bg._width-clip_drag._width))
		onVolumeUpdated()
	}
	
	/**
	* Broadcasts a message with the new volume
	* @return Void
	*/
	private function onVolumeUpdated():Void {
		Broadcaster.broadcastMessage("onVolumeUpdated",[volume_int])
	}
	
	//LISTENER BROADCASTER
	public function addListener(obj:Object):Boolean {
		return( Broadcaster.addListener(obj) )
	}
	
	public function removeListener(obj:Object):Boolean {
		return( Broadcaster.removeListener(obj) )
	}

}