import com.fwa.ui.Node

/**
 * A class used for managing the depths for movieclips.
 * This class relies heavy on linked structures for efficiency and speed.
 *
 * @author Michael Avila
 * @version 1.1.0
 * @modified 2006/09/27
 */
class com.fwa.ui.DepthManager {
	private static var startDepth:Number = 100; // The depth that this manager will begin allocating at
	
	private var firstNode:Node = null; // The first node in the list
	private var lastNode:Node = null; // The last node in thelist
	private var lastModified:Node = null // Last Modified Node ( also the position )
	private var depthOffset:Number = 0; // the offset for the depth... we need this offset because we are no longer applying depths from the firstNode
	private var _autoApply:Boolean = true; // Whether or not to apply the depths to the movieclips after each change
	private var beenApplied:Boolean = false; // Whether or not the depths have been applied yet
	private var needsApply:Boolean = false; // Whether or not the depths need to be applied

	// Gets and sets the auto apply property
	public function get autoApply ():Boolean { return _autoApply;	}
	public function set autoApply (apply:Boolean) { _autoApply = (apply == null) ? false : apply; }
	
	/**
	* Constructor
	*/
	public function DepthManager (autoApplyDepths:Boolean){
		if (autoApplyDepths != undefined || autoApplyDepths != null) {
			_autoApply = autoApplyDepths;
		}
	}
	
	/**
	* Returns whether this manager is empty or not
	*/
	public function isEmpty ():Boolean 	{
		return ( (firstNode==null || firstNode==undefined) );
	}
	
	/**
	* Find and occupy the depth before the specified clip, if the clip you specified does not exist within 
	* the depth manager the clip will be placed at the beginning.
	*/
	public function occupyBefore(beforeClip:MovieClip, clip:MovieClip) {
		if(beforeClip !== clip){
			//check if the given clip does not already exists in the Nodes Chain
			var existant:Node = getClipNode(clip)
			var nodeExists:Boolean = (existant!==undefined)
			var newNode:Node = (nodeExists==false)? new Node(clip, null) : existant
			
			if (isEmpty()){
				firstNode = newNode;
				lastNode = newNode;
				debug("occupyBefore", "creating first/last node")
			}else{
				//Ajustar primero la referencia actual al newNode (su parent)
				if(nodeExists){
					//SI EL NODO YA EXISTIA
					//buscar el parent del Node y modificar su referencia Next
					var newNodeParent:Node = getNodeParent(newNode)
					debug("occupyBefore", "newNodeParent:"+newNodeParent.clip_name)
					//si el parent del newNode es el mismo, ese nodo es firstNode
					if(newNodeParent == newNode){
						debug("occupyBefore","newNodeParent is firstNode")
						firstNode = firstNode.next //reasignar firstNode
					}else{
						newNodeParent.next = newNode.next //reasignar el next del Parent
						debug("occupyBefore","newNodeParent's next updated")
					}
					
					//si newNode era lastNode, hacer que su Parent lo sea ahora
					if(newNode == lastNode){
						lastNode = newNodeParent
					}
					
				}
				
				//Ajustar luego la referencia a/del beforeNode (de su parent, y de su next)
				var beforeNode:Node = getClipNode(beforeClip)
				var beforeParent:Node = getNodeParent(beforeNode)
				
				debug("occupyBefore","beforeNode: "+beforeNode.clip_name)
				debug("occupyBefore","beforeParent: "+beforeParent.clip_name)
				
				newNode.next = beforeNode
				if(beforeNode == beforeParent){
					//si before es firstNode
					firstNode = newNode
					debug("occupyBefore","beforeNode is firstNode!")
				}else{
					debug("occupyBefore", "beforeParent's next updated")
					debug("occupyBefore", "               old:"+beforeParent.next)
					beforeParent.next = newNode
					debug("occupyBefore", "               new:"+beforeParent.next)
				}
			}
			
			updateDepths(newNode)
		}else{
			debug("occupyBefore", "WARNING: beforeClip and clip are already the same")
		}
	}
	
	public function occupyBefore_(beforeClip:MovieClip, clip:MovieClip) {
		if(beforeClip !== clip){
			var existant:Node = getClipNode(clip)
			var nodeExists:Boolean = (existant!==undefined)
			var newNode:Node = (nodeExists==false)? new Node(clip, null) : existant
			var beforeNode:Node = getClipNode(beforeClip)
			var beforeParent:Node = getNodeParent(beforeNode)
			
			//si no existe firstNode, o no se definio beforeClip o beforeNode es firstNode
			if(isEmpty() || beforeClip==undefined || beforeNode == firstNode){
				debug("occupyBefore", "WARNING: sending First")
				debug("occupyBefore", "isEmpty: "+isEmpty())
				debug("occupyBefore", "beforeClip is undefined: "+(beforeClip==undefined))
				debug("occupyBefore", "beforeNode is firstNode: "+(beforeNode == firstNode))
				occupyFirst (clip);
			}else{
				debug("occupyBefore", "newNode:"+newNode.clip_name)
				debug("occupyBefore", "beforeNode:"+beforeNode.clip_name)
				debug("occupyBefore", "beforeParent:"+beforeParent.clip_name)
				
				//chequear que el nuevo nodo que estoy moviendo no sea firstNode o lastNode
				if(newNode == firstNode){
					firstNode = firstNode.next
				}
				if(newNode == lastNode){
					lastNode = getNodeParent(lastNode)
				}
				
				//coloco el nuevo nodo a continuacion del padre del nodo "before"
				if(beforeParent!==undefined){ beforeParent.next = newNode } 
				newNode.next = beforeNode //coloco "before" a continuacin del nuevo nodo
				
				updateDepths(newNode)
			}
		}else{
			debug("occupyBefore", "WARNING: beforeClip and clip are already the same")
		}
	}

	/**
	* Find and occupy the depth after the specified clip, if the clip you specified does not exist within the depth manager
	* the clip will be placed at the end.
	*/
	public function occupyAfter(afterClip:MovieClip, clip:MovieClip):Void {
		if(afterClip!==clip){
			//check if the given clip does not already exists in the Nodes Chain
			var existant:Node = getClipNode(clip)
			var nodeExists:Boolean = (existant!==undefined)
			var newNode:Node = (nodeExists==false)? new Node(clip, null) : existant
			
			debug("occupyAfter", "clip exists? "+nodeExists)

			if (isEmpty()){
				firstNode = newNode;
				lastNode = newNode;
				debug("occupyAfter", "creating first/last node")
			}else{
				if(nodeExists){
					//SI EL NODO YA EXISTIA
					//buscar el parent del Node y modificar su referencia Next
					var newNodeParent:Node = getNodeParent(newNode)
					
					//si el parent del newNode es el mismo, ese nodo es firstNode
					if(newNodeParent == newNode){
						firstNode = firstNode.next //reasignar firstNode
					}else{
						newNodeParent.next = newNode.next //reasignar el next del Parent
					}
				}
				
				//Una vez modificada la referencia del parent, agregar este nodo como Next de AfterNode
				//y hacer que el actual next de afterNode sea el next de newNode
				var afterNode:Node = getClipNode(afterClip)
				
				//si afterNode era el ultimo (lastNode), modificar la referencia de lastNode
				if(lastNode == afterNode){ lastNode = newNode }
				
				newNode.next = afterNode.next
				afterNode.next = newNode
					
				debug("occupyLast", "new lastNode:"+afterNode.clip_name)
				debug("occupyLast", "lastNode Next:"+afterNode.next.clip_name)
				debug("occupyLast", "lastNode Parent:"+getNodeParent(afterNode).clip_name)
				trace("..................................................................")
			}
			
			updateDepths(newNode)
		}else{
			debug("occupyAfter", "WARNING: afterClip and clip are already the same!")
		}
	}
	
	/**
	* Occupy the first depth
	*/
	public function occupyFirst(clip:MovieClip):Void {
		if(clip!==undefined){
			occupyBefore(MovieClip(firstNode.data), clip)
		}else{
			debug("occypuFirst", "WARNING: undefined clip")
		}
	}
	
	/**
	* Find and occupy the highest depth
	*/
	public function occupyLast(clip:MovieClip):Void {
		if(clip!==undefined){
			occupyAfter(MovieClip(lastNode.data), clip)
		}else{
			debug("occupyLast", "WARNING: undefined clip")
		}
	}

	/**
	* Removes the clip from the manager
	*/
	public function remove (clip:MovieClip):Void {
		debug("remove", "removing clip:"+clip._name)
		var rNode:Node = getClipNode(clip)
		if(rNode!==undefined){
			//remove node's parent reference
			var rNodeParent:Node = getNodeParent(rNode)
			if(rNodeParent == rNode){
				//if this Node (already firstNode) has no Next
				//then it is the only one in the DepthManager -> update firstNode and lastNode
				if(rNode.next!==undefined && rNode.next!==null){
					firstNode = rNode.next
				}else{
					firstNode = null
					lastNode = null
				}
				
				debug("remove", "WARNING: Node removed was firstNode")
				if(lastNode == rNode){
					lastNode = null
					debug("remove", "WARNING: Node removed was lastNode")
				}
				
				updateDepths(firstNode)
			}else{
				rNodeParent.next = rNode.next
				debug("remove", "Node removes OK")
				//if this node was lastNode, then his parent will be lastNode
				if(lastNode == rNode){
					lastNode = rNodeParent
				}
				updateDepths(rNodeParent)
			}
		}else{
			debug("remove", "node ["+clip._name+"] doesnt appear to exist!")
		}
	}
	
	/**
	* Applys the model of depths, for each clip, to the actual depths of the clips
	*/
	public function applyDepths (){
		debug("applyDepths","init")
		// if we don't need to do this.. then don't
		//if ( !needsApply && _autoApply==false ) return;
		
		var startNode:Node = firstNode //(beenApplied && lastModified!==null)? lastModified : firstNode;
		var count:Number = 0;
		for (var node:Node = startNode; node != null; node = node.next){
			var newdepth:Number = DepthManager.startDepth + (count) // + depthOffset )
			node.data.swapDepths (newdepth);
			
			debug("applyDepths", count+"> clip:"+node.data._name+" depth:"+node.data.getDepth())
			
			count++;
		}
		
		needsApply = false;
		lastModified = lastNode;
		if (!beenApplied) beenApplied = true;
		//trace("........................................")
		//trace("..END...................................")
	}
	
	/**
	* Retrieves the Highest Node's Depth
	*/
	public function getHighestDepth():Number {
		var depth:Number = lastNode.data.getDepth() + 1
		return( (isNaN(depth)? 0 : depth) )
	}
	
	/** PRIVATE METHODS **/
	
	/**
	* Returns a clip's node
	*/
	private function getClipNode(clip:MovieClip):Node {
		for (var node:Node = firstNode; node != null; node = node.next){
			if(node.data == clip){
				debug("getClipNode", "node found:"+node.clip_name)
				return(node)
			}
			if(node.next == node.next.next){
				debug("getClipNode", "WARNING: recursive Node found:"+node.clip_name)
				//return(undefined)
			}
		}
		debug("getClipNode", "node NOT FOUND:"+clip._name)
		return(undefined)
	}
	
	/**
	* Returns a node's parent node
	*/
	private function getNodeParent(node:Node):Node {
		//check if the node's parent is firstNode
		for (var pnode:Node = firstNode; pnode != null; pnode = pnode.next){
			debug("getNodeParent", "searching:"+ node.data+"/"+pnode.next.data)
			if(pnode.next.data == node.data){
				//debug("getNodeParent", "parent node found:"+ pnode.clip_name)
				return(pnode)
			}
			if(pnode.next == pnode.next.next){
				debug("getNodeParent", "WARNING: recursive Node found:"+pnode.clip_name)
				//return(undefined)
			}
		}
		//debug("getNodeParent", "parent node NOT FOUND")
		//si lleg hasta aca, chequear que el parent no sea firstNode
		if(firstNode.data == node.data){
			return(firstNode)
		}else{
			return(undefined)
		}
	}
	
	/**
	* Updates all depths
	*/
	private function updateDepths(node:Node):Void {
		lastModified = node
		if (_autoApply){
			applyDepths();
		}else{
			needsApply = true;
		}
	}
	
   /** DIAGNOSTICS **/
	public function showAll (){
		trace("firstNode:"+firstNode.clip_name)
		for (var node:Node = firstNode; node != null; node = node.next){
			trace (node.data);
		}
		trace("lastNode:"+lastNode.clip_name)
	}
	
	private function debug(meth:String, msg:String){
		//trace("DepthManager >"+meth+" >"+msg)
	}
}