/**
* This class extends the intrinsic Mouse functionality
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.1
* @date 	28 Nov 2006
*/
class com.fwa.utils.MouseExtension extends Mouse {
	
	private static var blocker:MovieClip = null;
	public static var onClick:Function;
	
	
	/**
	* Enables the blocker clip. If no blocker clip was set, creates one in _level0 highest depth.
	* @return	Void
	*/
	public static function block():Void {
		if(blocker==null){
			blocker = createBlocker()
			blocker.onRelease = onClick;
			blocker.onRollOver = function(){
				Mouse.hide()
			}
			blocker.onReleaseOutside = blocker.onRollOut = function(){
				Mouse.show()
			}
			//blocker.enabled = false
			blocker.useHandCursor = false
		}
	}
	
	/**
	* Disables the blocker clip (if any)
	* @param	remove (Boolean) if true, removes the blocker from stage
	* @return	Void
	*/
	public static function unblock(remove:Boolean):Void {
		if(blocker!==null){
			delete blocker.onRelease;
			delete blocker.onRollOver;
			delete blocker.onRollOut;
			delete blocker.onReleaseOutside;
			if(remove==true){
				blocker.removeMovieClip()
				blocker =null
			}
		}
	}
	
	/**
	* Returns true if a blocker is set
	* @return	Boolean
	*/
	public static function isBlocked():Boolean {
		return((blocker==null)? false : true)
	}
	
	/**
	* Allows the user to set a custom MovieClip as blocker
	* @param	clip (MovieClip) Clip to use as blocker
	* @return	Void
	*/
	public static function setBlocker(clip:MovieClip):Void {
		blocker = clip
	}
	
	/**
	* Creates a MovieClip in _level0 and returns its reference.
	* The created clip is filled to fit the Stage's size, and transparent (_alpha = 0)
	* @return	MovieClip
	*/
	private static function createBlocker():MovieClip {
		var blk:MovieClip = _level0.createEmptyMovieClip("blocker", _level0.getNextHighestDepth());
		blk.beginFill(0xFFFFFF, 0);
		blk.moveTo(0,0);
		blk.lineTo(Stage.width, 0);
		blk.lineTo(Stage.width, Stage.height);
		blk.lineTo(0, Stage.height);
		blk.lineTo(0,0);
		blk.endFill();
		
		return(blk)
	}
}