import com.fwa.system.FWAConfiguration;
import com.fwa.data.NodeRetriever;

/**
* Static class Accesibilidad
* This class works as an API to retrieve the content of the Accesbility.xml nodes.
* This Class also allows to apply Accesibility Properties to a given MovieClip.
* 
* @version 	1.2
* @since 	02 Dec 2006
*/

/**
* @ignore
* VERSION HISTORY
* 
* 1.1 - 17 Dec 2006
* + Renamed from "Miscelaneas.as" to "Accesibilidad.as"
* + Added methods to apply accesibility to a MovieClip
* 
* 1.2 - 19 Mar 2007
* + Updated the makeAccesible method
* + Added a counter for the tabIndex property
*/
class com.fwaapi.Accesibilidad {
	
	//PROPERTIES
	private static var tabIndexCount:Number = 0;
	
	
	/**
	* Hidden Constructor 
	*/
	private function Accesibilidad(){}
	
	/**
	* This method returns a String with the content of the given Accesibility Node identifier
	* @param	identifier (String) Identifier of the content to retrieve
	* @return	String
	*/
	public static function getLabel(identifier:String):String {
		var node:Object = getNode(identifier)
		return(node.name._value)
	}
	
	/**
	* This method returns a String with the description of the given Accesibility Node identifier
	* @param	identifier (String) Identifier of the description to retrieve
	* @return	String
	*/
	public static function getDescription(identifier:String):String {
		var node:Object = getNode(identifier)
		return(node.description._value)
	}
	
	/**
	* Makes the provided target object accesible using the labels in Accesibility.xml 
	* configuration under the given identifier.
	* Returns true if the target was successfully made Accessible (config found in XML).
	* @param	target (Object) Object to make accesible (Button, MovieClip, TextField)
	* @param	identifier (String) Accesibility content identifier
	* @return	Boolean
	*/
	public static function makeAccessible(target:Object, identifier:String):Boolean {
		var node:Object = getNode(identifier)
		var msg:String = (node.description._value.length>0)? node.description._value : node.name._value
		
		if(node!==undefined){
			tabIndexCount += 1;
			target._accProps = new Object({name:msg, forceSimple:true});
			target.tabIndex = tabIndexCount;
			Accessibility.updateProperties();
			return(true)
		}else{
			trace("ACCESSIBILITY NODE NOT FOUND!")
			return(false)  
		}
	}
	
	
	/**
	* Makes the provided target object unaccessible for Screen Readers
	* @param	target (Object) Object to make unaccessible
	*/
	public static function makeUnaccessible(target:Object):Void {
		target._accProps = new Object({silent:true, forcesimple:true});
	}
	
	
	
	//PRIVATE METHODS
	/**
	* Retrieves the given identifier's node in the Accesibility.xml System File
	* @param	identifier (String) Identifier of the node whose content needs to be retrieved
	* @return	Object
	*/
	private static function getNode(identifier:String):Object {
		var obj:Object = FWAConfiguration.accesibility
		var node:Object = NodeRetriever.retrieveFirstNode(identifier, obj.label)
		
		return( node )
	}
}