import com.fwa.system.FWAConfiguration;
import com.fwa.loaders.FilesLoader;
import com.fwa.data.NodeRetriever;
import com.fwa.debug.Tracer;
import mx.utils.Delegate;

/**
* Static class Resources
* This class provides methods to load and preload FWA Resources.
* 
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.0
* @date		28 Nov 2006
*/
class com.fwaapi.Resources {
	//properties
	public static var onProcessStart:Function;
	public static var onProcessProgress:Function;
	public static var onProcessComplete:Function
	
	/**
	* Constructor
	*/
	private function Resources(){}
	
	
	/**
	* Creates a preload job with all the resources passed in the identifiers array.
	* After the preload job is completed, the endFunction is called in the given scope, with the given params.
	* 
	* @param	identifiers (Array) List of resources identifiers to preload
	* @param	endFunc (Function) Function to invoke when all identifiers are preloaded
	* @param	endArgs (Array) Arguments to pass to the end function
	* @param	endScope (Object) Scope of the end function
	* @return	Void
	*/
	public static function preload(identifiers:Array, endFunc:Function, endArgs:Array, endScope:Object):Void {
		endScope = (endScope == undefined)? null : endScope
		endArgs = (endArgs == undefined)? null : endArgs
		
		var fl:FilesLoader = new FilesLoader()
		var count:Number = identifiers.length;
		
		//retrieve all identifiers and push the files into the SoundLoader
		for(var i:Number = 0; i<count; i++){
			var s_file:String = getResourcesPath() + getFile(identifiers[i]);
			fl.addFile(s_file);
		}

		var sl_lst:Object = new Object();
		sl_lst.onProcessComplete = function(){
			endFunc.apply(endScope, endArgs)
			Resources.onProcessComplete()
		}
		sl_lst.onProcessProgress = Delegate.create(Resources, Resources.onProcessProgress)
		sl_lst.onProcessStart = Delegate.create(Resources, Resources.onProcessStart)
		fl.addListener(sl_lst)
		fl.process()
	}
	
	
	/**
	* Retrieves a resource identifier's associated file
	* @param	identifier (String) identifier of the resource
	* @return	String
	*/
	public static function getFile(identifier:String):String {
		var node:Object = getNode(identifier)
		return(node.file._value)
	}
	
	
	
	
	//PRIVATE METHODS
	/**
	* Retrieves the full path to the Resources folder.
	* @return	String
	*/
	private static function getResourcesPath():String {
		var app_path:String = FWAConfiguration.application+FWAConfiguration.language
		var rsc:Object = NodeRetriever.retrieveFirstNode("resources", FWAConfiguration.configuration.application.paths.path)
		var files_path:String = app_path+"/"+rsc.folder._value+"/"
		return(files_path)
	}

	/**
	* Retrieves the given identifier's node.
	* @param	identifier (String) Identifier of the node that wants to be retrieved
	* @return	Object
	*/
	private static function getNode(identifier:String):Object {
		var obj:Object = FWAConfiguration.resources
		var node:Object = NodeRetriever.retrieveFirstNode(identifier, obj.resource)
		
		return( node )
	}
}