/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.condition.*;

class asdk.AbstractMap implements Map
{	
  private var comparator : BinaryCondition;
  
  // abstract
  function getSize() : Number
  {
	  throwError();
	  return -1;
  }
  
  // abstract
  function get( key )
  {
	  throwError();
	  return undefined;
  }
	
  // abstract	
  function put( key, value )
  {
	  throwError();
  }

  // abstract
  function remove( key )
  {
	  throwError();
  }
  
  // abstract
  function getKeys() : Set
  {
	  throwError();
	  return undefined;
  }
  
  // abstract
  function getValues() : Collection
  {
	  throwError();
	  return undefined;
  }  

  // abstract
  function iterator( type : Number ) : Iterator
  {
	  throwError();
	  return undefined;
  }  
  
  function setComparator( condition : BinaryCondition )
  {
	  this.comparator = condition;
  }
  
  function getComparator() : BinaryCondition
  {
	  if( this.comparator == undefined )
	  	this.comparator = new ObjectsEqual();
		
	  return this.comparator;
  }
	
  function isEmpty() : Boolean
  {
	var keys : Set = getKeys();
	return keys.isEmpty();
  }
	
  function containsKey( key ) : Boolean
  {
	var keys : Set = getKeys();
	return keys.containsItem( key );
  }
	
  function containsValue( value ) : Boolean
  {
	var values : Collection = getValues();
	return values.containsItem( value );
  }	  
 
  function putAll( map : Map ) : Void
  {
	  var keyIterator : Iterator = map.getKeysIterator();
	  
	  while( keyIterator.hasNext() )
	  {
		  var key = keyIterator.getNext();
		  put( key, map.get( key ) );
	  }	  
  }
  
  function clear() : Void
  {
	  var keyIterator : Iterator = getKeysIterator();
	  
	  while( keyIterator.hasNext() )
	  {
		  var key = keyIterator.getNext();
          keyIterator.remove();
	  }	  	  
  }
  
  function getKeysIterator() : Iterator
  {
	  return getKeys().iterator();
  }
  
  function getValuesIterator() : Iterator
  {
	  return getValues().iterator();
  }
  
  function throwError()
  {
	  throw new Error( "AbstractCollection: this method is abtract and not implemented" );
  }
}