/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.*;
import asdk.condition.*;

class asdk.algorithm.Finding
{
	static function find( collection : Collection, object, comparator : BinaryCondition ) : Iterator
	{
		if( comparator == undefined )
			comparator = new ObjectsEqual();
			
		var iteratorObj : Iterator = collection.iterator();
		
		while( iteratorObj.hasNext() )
			if( comparator.execute( iteratorObj.getNext(), object ) )
			     return iteratorObj;
		
		return undefined;
	}
	
	static function findIf( collection : Collection, condition : UnaryCondition ) : Iterator
	{
		var workerIterator : Iterator = collection.iterator();
		var iteratorObj : Iterator = collection.iterator();
		
		while( workerIterator.hasNext() )
		{
			if( condition.execute( workerIterator.hasNext() ) )
				return iteratorObj;
			
			iteratorObj.getNext();
		}
		
		return undefined;
	}
	
	static function detect( collection : Collection, condition : UnaryCondition )
	{
		var iteratorObj : Iterator = collection.iterator();
		
		while( iteratorObj.hasNext() )
		{
			var object = iteratorObj.getNext();
			
			if( condition.execute( object ) )
				return object;
		}
		
		return undefined;
	}
	
	static function every( collection : Collection, condition : UnaryCondition ) : Boolean
	{
		var iteratorObj : Iterator = collection.iterator();
		
		while( iteratorObj.hasNext() )
		{
			var object = iteratorObj.getNext();
			
			if( !condition.execute( object ) )
				return false;
		}
		
		return true;
	}
	
	static function some( collection : Collection, condition : UnaryCondition ) : Boolean
	{
		var iteratorObj : Iterator = collection.iterator();
		
		while( iteratorObj.hasNext() )
		{
			var object = iteratorObj.getNext();
			
			if( condition.execute( object ) )
				return true;
		}
		
		return false;
	}
}