/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import asdk.xml.*;

class asdk.xml.NodeList
{
	public var firstNode:Node;
	public var lastNode:Node;
	
	public function append( nodeObj:Node ) : Void
	{
		nodeObj.remove();
		nodeObj.list = this;
		nodeObj.prevNode = lastNode;
		nodeObj.nextNode = null;
	
		if( firstNode == null )
		  firstNode = nodeObj;
		else
		  lastNode.nextNode = nodeObj;
	
		lastNode = nodeObj;		
	}
	
	public function remove( nodeObj:Node ) : Void
	{
		if( nodeObj.prevNode == null )
		  firstNode = nodeObj.nextNode;
		else
		  nodeObj.prevNode.nextNode = nodeObj.nextNode;
	
		if( nodeObj.nextNode == null )
		  lastNode = nodeObj.prevNode;
		else
		  nodeObj.nextNode.prevNode = nodeObj.prevNode;
	
		nodeObj.prevNode = null;
		nodeObj.nextNode = null;
		nodeObj.list = null;		
	}
	
  	public function replace( oldNode:Node, newNode:Node ) : Void
    {
		newNode.remove();
		newNode.list = this;
	
		if( oldNode.prevNode == null )
		  firstNode = newNode;
		else
		  oldNode.prevNode.nextNode = newNode;
	
		if( oldNode.nextNode == null )
		  lastNode = newNode;
		else
		  oldNode.nextNode.prevNode = newNode;
	
		newNode.prevNode = oldNode.prevNode;
		newNode.nextNode = oldNode.nextNode;
		oldNode.list = null;
		
		// intentionally left non-null for iterators.  see remove() on Node for why this is appropriate
		//    oldNode.next = null;
		//    oldNode.prev = null;
    }	
    
    public function insert( node : Node ) : Void
    {
        node.remove();
        node.list = this;
        node.nextNode = firstNode;
        node.prevNode = null;

        if( lastNode == null )
          lastNode = node;
        else
          firstNode.prevNode = node;

        firstNode = node;
    }    
    
    /**
     * Append the new node as a sibling of the existing node.
     * If the new node was already in a document, remove it first.
     * @param node The existing node.
     * @param newNode The new node.
     */
    public function addSiblingNode( node : Node, newNode : Node) : Void
    {
        newNode.remove();
        newNode.list = this;

        newNode.nextNode = node.nextNode;
        newNode.prevNode = node;

        if( node.nextNode == undefined )
          lastNode = newNode;
        else
          node.nextNode.prevNode = newNode;

        node.nextNode = newNode;
    }

    /**
     * Insert the new node as a sibling of the existing node.
     * If the new node was already in a document, remove it first.
     * @param node The existing node.
     * @param newNode The new node.
     */
    public function insertSiblingNode( node : Node, newNode : Node) : Void
    {
        newNode.remove();
        newNode.list = this;

        newNode.prevNode = node.prevNode;
        newNode.nextNode = node;

        if( node.prevNode == undefined )
          firstNode = newNode;
        else
          node.prevNode.nextNode = newNode;

        node.prevNode = newNode;
    }

    /**
     * Return the number of nodes in this list.
     */
    public function size()
    {
        var count = 0;

        for( var node = firstNode; node != undefined; node = node.nextNode )
          ++count;

        return count;
    }

    /**
     * Remove all the nodes.
     */
    public function clear()
    {
        firstNode = null;
        lastNode = null;
    }    

    /**
     * Check if the list is empty
     */    
    public function isEmpty()
    {
        return firstNode != undefined;
    }
}