/*
Copyright 2005 Midnight Coders, LLC

Licensed under the Midnight Coders license (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.themidnightcoders.com/licensing/asdklicense.htm

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/


import asdk.xml.*;
import mx.data.components.*;

class asdk.xml.XMLLoader
{
	var XML_NODE_TYPE_TEXT:Number = 3;
	var XML_NODE_TYPE_ELEMENT:Number = 1;
	var callBackFN : String;
	var callBackObj:Object;
	var metadata;
	var object;
	
	var xmlConnector ;
	var xmlListener : XMLListener;
	
	function XMLLoader( callbackOwner, fpHandlerFunction : String )
	{
		this.callBackFN = fpHandlerFunction;
		this.callBackObj = callbackOwner;
		this.xmlConnector = new XMLConnector();
		xmlConnector.ignoreWhite = true;
		xmlConnector.hostObject = this;
		xmlListener = new XMLListener( this );
		xmlConnector.addEventListener( "status", xmlListener.status );
		xmlConnector.addEventListener( "send", xmlListener.send );
		xmlConnector.addEventListener( "result", xmlListener.result );
		xmlConnector.addEventListener( "receive", xmlListener.receive );
	}
	
	function sendXML( url: String, params : XML, metadata )
	{
		xmlConnector.direction = "send";
		xmlConnector.params = params;
		xmlConnector.URL = url;
		this.metadata = metadata;
		xmlConnector.trigger();
	}
	
	function receiveXML( url: String, metadata )
	{
		xmlConnector.direction = "receive";
		xmlConnector.URL = url;
		this.metadata = metadata;
		xmlConnector.trigger();
	}
	
	function sendReceiveXML( url: String, params : XML, metadata )
	{
		xmlConnector.direction = "send/receive";
		xmlConnector.URL = url;
		xmlConnector.params = params;
		this.metadata = metadata;
		xmlConnector.trigger();
	}
	
	function receive( evt )
	{
		var parent = _parent;
		_root.startDebugging();
		
	}
	
	function receiveResults( result : XML )
	{
		var asData = xmlDocToObject( result );
		callBackObj[ callBackFN ]( asData, metadata );
	}
	
	
	public function getObject()
	{
		return this.object;
	}
	
	public function xmlDocToObject( oXML )
	{
		
		var obj:Array = new Array();
		if( oXML == null )
			return obj;

		var eRoot = oXML.firstChild;
		
		if( eRoot != null )
			obj = this.buildObject( oXML.originalRef, obj, eRoot );
		
		return obj;
	}
	
	public static function transformDocToObject( xmlDoc )
	{
		var xmlLoader : XMLLoader = new XMLLoader();
		return xmlLoader.xmlDocToObject( xmlDoc );
	}
	
	private function buildObject( container:MovieClip, obj, eItem )
	{
		var idx:XMLNode, eChild;
		var oTarget:Array;
		
		while( eItem != null )
		{
			
			idx = eItem.nodeName;
			if( eItem.nodeType == this.XML_NODE_TYPE_ELEMENT )
			{
				var path = this.setPathFromContainer( container, idx );
	
				//--- Recursively process any other child nodes ---
				oTarget = this.buildObject( path, {}, eItem.firstChild );
	
				//--- Process any XML node attributes ---
				for( var attrib:String in eItem.attributes )
					oTarget[ attrib ] = eItem.attributes[ attrib ];
	
				//--- Check the first child, and see if it's a simple text node ---
				//--- Save the value from the TEXT element into the object ---
				if( eItem.nodeValue != null )
					oTarget._value = eItem.nodeValue;
				else
					this.setChildValue( eItem, oTarget )
	
	
				//--- If there are duplicate nodenames, convert them to an array ---
				if( obj[ idx ] != null )
					this.establishObjArray( obj, idx, oTarget );
				else
				{
					var objPath = container + "_" + idx;
					var movie = eval( objPath );
					if( typeof( movie ) == 'movieclip' )
					{
						if( oTarget._value == undefined )
						{
							movie.setNewData( oTarget );
						}
						else
						{
							movie.setNewData( oTarget._value );
						}
					}
					else if( typeof ( oTarget._value ) == 'string' )
					{
						set( objPath, oTarget._value );
					}
					obj[ idx ] = oTarget;
				}
			}
				//*** END OF: if (eItem.nodeType == XML_NODE_TYPE_ELEMENT) ***
			//-- Get the next sibling node in this level of the XML ---
			eItem = eItem.nextSibling;
		}
		//*** END OF: while(eItem != null) ***
		return obj;
	}
	
	private function setPathFromContainer( container:MovieClip, idx ):String
	{
		if( typeof ( container ) == 'movieclip' )
			return container + "." + idx;
		else
			return container + "_" + idx;
	}
	
	private function setChildValue( eItem, oTarget )
	{
		var eChild = eItem.firstChild;
		if( eChild != null )
		{
			if( eChild.nodeType == this.XML_NODE_TYPE_TEXT )
			{
				if( eChild.nodeValue != null )
				{
					//--- Save the value from the TEXT element into the object ---
					// This nodeValue has the failed requests for the services portion - Eran Kriegshauser
					oTarget._value = eChild.nodeValue;
				}
			}
		}
	}
	
	private function establishObjArray( obj, idx, oTarget )
	{
		if( obj[ idx ]._type != 'array' )
		{
			obj[ idx ] = [ obj[ idx ] ];
			obj[ idx ]._type = "array";
			obj[ idx ][ 1 ] = oTarget;
		}
		else
			obj[ idx ][ obj[ idx ].length ] = oTarget;
	}
}
