import com.fwa.controllers.ScrollBar;
import mx.utils.Delegate;

/**
* This class contains the functionality of a ScrollPane component.
* 
* @version	1.0
* @date 	9 Dec 2006
*/
class com.fwa.controllers.ScrollPane extends MovieClip {
	//elements
	private var mask:MovieClip;
	private var content:MovieClip;
	private var scrollBar:MovieClip;
	
	//properties
	private var contentMaxPos:Number;
	
	public function ScrollPane(){
	}
	
	/**
	* Initialises the ScrollPane
	*/
	public function init():Void {
		var scr:ScrollBar = ScrollBar(scrollBar);
		var scr_lst:Object = new Object();
		scr_lst.onUpdate = Delegate.create(this, scrollContent);
		
		scr.addListener(scr_lst)
	}
	
	/**
	* Attaches an item to the ScrollPane content and returns a reference to the new Item.
	* @param	linkage (String) Linkage name of the item to attach
	* @param	name (String) Name of the clip to attach
	* @param	xpos (Number) Target x position where to attach it
	* @param	ypos (Number) Target y position where to attach it
	* @return	MovieClip
	*/
	public function addItem(linkage:String, name:String, xpos:Number, ypos:Number):MovieClip {
		var newItem:MovieClip = content.attachMovie(linkage, name, content.getNextHighestDepth())
		newItem._x = xpos
		newItem._y = ypos
		update()
		return(newItem)
	}
	
	
	public function removeAll():Void {
		for(var i:String in content){
			content[i].removeMovieClip()
		}
		content._y = 0
		update()
	}
	
	/**
	* Returns the current content's height
	* @return	Number
	*/
	public function getHeight():Number {
		return(content._height)
	}
	
	/**
	* Updates the ScrollBar properties according to the ScrollPane's content
	* @return	Void
	*/
	private function update():Void {
		content._y = 0
		scrollBar.reset()
		var maskSize:Number = mask._height
		var contSize:Number = content._height + 5
		var percentMasked:Number = maskSize*100/contSize
		contentMaxPos = -(contSize)+maskSize
		
		if(maskSize<contSize){
			scrollBar.setSliderSize(Math.floor(percentMasked))
			scrollBar._visible = true
		}else{
			scrollBar._visible = false
		}
	}
	
	/**
	* This method is called every time the ScrollBar is updated.
	* It performs the content scroll action
	* @param	sliderPercPos (Number) ScrollBar's percent position of the slider
	* @return	Void
	*/
	private function scrollContent(sliderPercPos:Number):Void {
		var contentFinalPos:Number = Math.round(sliderPercPos * contentMaxPos /100)
		//content._y = Math.ceil(contentFinalPos)
		content.tween("_y", contentFinalPos, 0.5, "easeOutQuint")
	}
	
	
}