/**
* @class	Tracer
* This class provides methods to trace messages in the output window (or anywhere else)
* It formats the output message so you can specify which class, method or even package outputs the given message
* You can extend this class and override the parseMessage and outputMessage methods to customize the message's 
* parsing and/or ouput 
* 
* @author	Federico Campo Piombi
* @version	1.1
* @date		01 Nov 2006
*/

/**
* @ignore
* MODIFICATION HISTORY
* 
* v1.1 - 11 Nov 2006
* Added the parseMessage method
* Added the outputMessage method
* Intended to make this class extensible so you can customize this methods with your own functionality
* 
* @ignore
*/
class com.fwa.debug.Tracer {
	
	/**
	* @constructor
	*/
	private function Tracer(){}
	
	/**
	* @method	output
	* This method works as a bridge between the parseMessage and outputMessage methods.
	* 
	* @param	msg (String) Message to output
	* @param	method (String) Method that ouputs the message
	* @param	classs (String) Class that outputs the message
	* @param	pack (String) Package that outputs the message
	*/
	public static function output(msg:String, method:String, classs:String, pack:String):Void {
		var msg_str:String = parseMessage(msg, method, classs, pack)
		outputMessage(msg_str)
	}
	
	/**
	* @method	parseMessage
	* Returns a String with a custom formatted message using the message arguments
	* 
	* @param	msg (String) Message to output
	* @param	method (String) Method that ouputs the message
	* @param	classs (String) Class that outputs the message
	* @param	pack (String) Package that outputs the message
	* @return	String
	*/
	private static function parseMessage(msg:String, method:String, classs:String, pack:String):String {
		var o_str:String = ""
		o_str += (pack!==undefined)? "("+pack+") " : ""
		o_str += (classs!==undefined)? "["+classs+"] " : ""
		o_str += (method!==undefined)? "<"+method+"> " : ""
		o_str += msg
		return(o_str)
	}
	
	/**
	* @method 	outputMessage
	* Outputs a sting in Flash's Output Panel
	* 
	* @param	msg (String) Message to output
	* @return	Void
	*/
	private static function outputMessage(msg:String):Void {
		trace(msg)
	}
}