﻿import com.fwa.events.IListenable

/**
* @class	ListenerBroadcaster
* This class provides integration to the ListenerBroadcaster event model by providing methods to store
* and remove Listeneres and to broadcast messages to them
* @author	Federico Campo Piombi [federicocp@hotmail.com]
* @version	1.1
* @date 	05 Jun 2005
*/

/**
* @ignore
* MODIFICATION HISTORY
* 
* v1.1 - 28 Oct 2006
* This class now implements the IListeneable Interface
* @ignore
*/
class com.fwa.events.ListenerBroadcaster implements IListenable {
	/**
	* @property _listeners
	* Stores a list of objects (Listeners)
	* @type  	Array
	*/
	private var _listeners:Array
	
	/**
	* Constructor
	*/
	public function ListenerBroadcaster(){
		this._listeners = new Array()
	}
	
	/**
	* Adds an object to the _listeners Array. Prior to adding the new object, removes any existant
	* instance of that object in the _listeners list to avoid several calls to the same object.
	* Returns true by default.
	* @param	obj (Object) Listener to add to the list
	* @return	Boolean
	*/
	public function addListener(obj:Object):Boolean {
		this.removeListener(obj);
		this._listeners.push(obj);
		return true;
	}
	
	/**
	* Removes an object from the _listeners Array. Returns true if object was removed successfuly.
	* Returns false if the object was not found in the _listeners Array.
	* @param	obj (Object) Listener to remove from the list.
	* @return	Boolean
	*/
	public function removeListener(obj:Object):Boolean {
		for(var i=0; i<this._listeners.length; i++){
			if(this._listeners[i] == obj){
				this._listeners.splice(i,1);
				return true;
			}
		}
		return false;
	}
	
	/**
	* It crosses the _listeners Array and executes the <event> message (as a function call) and passes
	* the <params> Array as a list of parameters.
	* @param	event (String) Message to broadcast (function name to execute)
	* @param	params (Array) A list of arguments to pass to the function being executed
	* @return	Void
	*/
	public function broadcastMessage(event:String, params:Array):Void {
		for(var i=0; i<this._listeners.length; i++){
			this._listeners[i][event].apply(null,params)
		}
	}	
}