import com.fwa.system.gateway.IExternalData;
import com.fwa.data.XMLConstruct;
import mx.utils.Delegate;

/**
* @class 		ExternalData
* @implements 	IExternalData
* This class provides basic implementation to load external data files and parse them using XMLConstruct,
* and provides methods to access the loaded data, and an onLoad event to get notified when the data is fully loaded and parsed.
* 
* @author		Federico Campo Piombi [federicocp@hotmail.com]
* @version		1.1
* @date 		13 Nov 2006
*/

/**
* @ignore
* MODIFICATION HISTORY:
* 
* v1.1 - 14 Nov 2006
* Added the onLoadError event
*/
class com.fwa.system.gateway.ExternalData implements IExternalData {
	
	private var _filename:String	//stores the name of the file to load
	private var _data:Object		//stores an object with the parsed data
	
	/**
	* Constructor
	*/
	public function ExternalData(){
	}
	
	/**
	* Starts loading the filename file. If a filename is provided with this method, that filename will override
	* any previously set filename using the setter <filename>, otherwise the previously set fliename will be used.
	* If there's no filename specified the onLoadError callback will execute.
	* @param	inFilename (String) File to load
	* @return	Void
	*/
	public function load(inFilename:String):Void {
		if(filename==undefined){
			filename = inFilename
		}
		if(filename!==undefined){
			var ref = this
			var tXML:Object = new XMLConstruct();
			tXML.onLoad = function(success:Boolean){
				if(success){
					ref._data = this
					ref.onLoad(ref.data, ref.filename)
				}else{
					ref._data = undefined
					ref.onLoadError("Invalid or unavailable file: "+ref.filename)
				}
			}
			tXML.load(filename)
		}else{
			onLoadError("No filename specified")
		}
	}
	
	/**
	* Removes the stored _data object.
	* @return	Void
	*/
	public function clear():Void {
		_data = undefined
	}
	
	
	//GETTER-SETTERS
	public function get data():Object { return(_data) } //returns the _data object
	
	public function set filename(file:String):Void { _filename = file } //stores the filename to load
	public function get filename():String { return(_filename) } //retrieves the _filename property
	
	
	//EVENTS
	/**
	* Event executed when the external file is loaded an parsed
	* @param	data (Object) the loaded data object
	*/
	public function onLoad(data:Object):Void {}
	
	/**
	* Event executed when an error occurs trying to load the file
	* @param	err (String) Error description
	*/
	public function onLoadError(err:String):Void {}

}