﻿/**
* The Delegate class creates a function wrapper to let you run a function in the context
* of the original object, rather than in the context of the second object, when you pass a
* function from one object to another. (mx.utils.Delegate description)
* This implementation of the Delegate class allows the delegation of arguments along with function and scope.
* The cons of using this class instead of the mx.utils.Delegate, is that if the function/method that you are passing to
* is already passing its own arguments, they'll be overriden by the arguments being delegated.
* 
* Example:
* import com.fwa.utils.Delegate
* myObj.onData = Delegate.create(this, onDataReceived)
* 
* //assume that myObj executes an onData method, passing a <data> object as argument with the loaded data
* 
* private function onDataReceived(data:Object):Void {
* 	trace(data.username)
* 	//outputs: undefined
* 	//since the Delegate.create has specified no arguments to pass to onDataReceived, 
* 	//and those arguments overrides the original myObj.onData ones
* }
* 
* //For this kind of Delegation you should use the mx.utils.Delegate
* 
* 
* 
* @author	Federico Campo Piombi
* @version	1.0
* @date 	05 Jun 2005
* @see		mx.utils.Delegate
*/

/**
* @ignore
* MODIFICATION HISTORY
* @ignore
*/
class com.fwa.utils.Delegate {
	
	/**
	* Creates a functions wrapper for the original function so that it runs in the provided context.
	* To pass parameters along with the delegation, just add them as arguments for this method.
	* @parameter 	obj 	Context in which to run the function.
	* @paramater 	func 	Function to run.
	* @usage		Delegate.create(this, showMessage, "hello", "my name is Earl")
	* @return		Function
	*/
	public static function create(scope:Object, method:Function):Function	{
		var args:Array = arguments.slice (2, arguments.length);
		var f:Function = function () : Void {
			method.apply (scope, args);
		};
		return f;
	}
}
