﻿/**
* This class provides methods to validate and format different datatypes
* @author	Javier Alejandro Corra
* @version	1.0
* @date 	7 Jun 2006
*/
class com.wunderman.utils.Validate {
  /**
  * Constructor
  */
  private function Validate() {
  }


  // ###########################################################################
  // # STATIC METHODS
  // ###########################################################################

  /**
  * Returns true if the given string contains letters only
  * @param	str (String) input text
  * @return Boolean
  */
  public static function isChar(str:String):Boolean {
    var result = true;
    var i:Number = 0;
    var letter:String;

    while(i< str.length) {
      letter = str.substr(i, 1).toLowerCase();
      // Si no esta entre a-z  o es espacio
      if(!(letter>="a" && letter<="z") && (letter != " ")) {
        result = false;
      };
      i++;
    };

    return result && !Validate.isEmpty(str);
  }


  /**
  * Returns true if the given string contains numbers
  * @param	str (String) Input text
  * @return Boolean
  */
  public static function isNumber(str:String):Boolean {
    var result:Boolean = true;
    var i:Number = 0;
    var letter:String;

    while(i<str.length) {
      letter = str.substr(i, 1).toLowerCase();
      if(!(letter>="0" && letter<="9")) {
        result = false;
      };
      i++;
    };

    return result && !Validate.isEmpty(str);
  }

  /**
  * Returns true if the given string is a valid email address
  * @param	str (String) Input text
  * @return Boolean
  */
  public static function isEmail(str:String):Boolean {
    var result:Boolean = true;

    if(str == "") result = false;
    if(str.indexOf("@")==-1) result = false;
    if(str.indexOf(".")==-1) result = false;
    if(str.indexOf("@")!==str.lastIndexOf("@")) result = false;

    return result;
  }

  /**
  * Returns true if the given string is empty
  * @param	str (String) Input Text
  * @return Boolean
  */
  public static function isEmpty(str:String):Boolean {
   // return (Trim(str) == "");
   return (str == undefined) || (str.length < 1);
  }

  // Chequea el largo del contenido de un field. Le paso la longuitud minima
  // y un opcional de maxima
  /**
  * Performs a verification of a string's length.
  * Returns true if the string's length is between the min and max values provided.
  * @param	val (String) Input text
  * @param	min (Number) The minimum lenght allowed
  * @param	max (Number) The maximum length allowed
  * @return Boolean
  */
  public static function checkLength(val:String, min:Number, max:Number):Boolean {
    var len:Number = val.length;
    var rsp:Boolean = true;

    if(len < min) {
      rsp = false;
    } else if( (max != undefined) && (len > max) ) {
      rsp = false;
    };

    return rsp;
  }

  /**
  * Converts the given String into an empty String if it's undefined or contains the text "undefined"
  * @param	param (String) Input text
  * @return String
  */
  public static function safeParam(param:String):String {
	param = (trim(param) == "") ? "" : param
    param = (param == "undefined") ? "" : param;
    return (param == undefined) ? "" : param;
  }
  
  /**
  * Formats the given date with the given format "dd-mm-aaa"
  * @param	d (Number) The day
  * @param	m (Number) The month
  * @param	a (Number) The year
  * @param	format (String) the format
  * @return String
  */
  public static function formatDate(d:Number, m:Number, a:Number, format:String):String {
    var i:Number;
    var date:String = "";
    var fmt:Array = format.split("-");

    for(i=0; i<3; i++) {
      switch(fmt[i]) {
        case "dd":
          date += padNumber(d, 2);
          break;

        case "mm":
          date += padNumber(m, 2);
          break;

        case "aaaa":
          date += padNumber(a, 4);
          break;
      };//end-switch
    };//end-for

    return date;
  }


  /**
  * Adds zeros to the left of a number until it has "digits" digits and returns a String with it.
  * @param	num (Number) The number
  * @param	digits (Number) The number of digits
  * @return String
  */
  public static function padNumber(num:Number, digits:Number):String {
    var nstr:String = String(num);

    while(nstr.length < digits) {
      nstr = "0" + nstr;
    };

    return nstr;
  }
	
	/**
	* Removes the leading whitespaces chars of a String
	* @param	str (String) The input text
	* @return	String
	*/
	public static function trimLeft(str:String):String {
		var l:Number
		for (l=0; l < str.length; l++){
			   if ((str.charAt(l) != " ") && (str.charAt(l) != "\n") && (str.charAt(l) != "\r") && (str.charAt(l) != "\t"))
			   {
				 break;
			   }
		}
		return str.slice(l);
	}
	
	/**
	* Removes the trailing whitespaces chars of a String
	* @param	str (String) The input text
	* @return	String
	*/
	public static function trimRight(str:String):String {
		// trailing whitespace chars
		var t:Number
		for (t = str.length; t > 0; t--) {
			if ((str.charAt(t) != " ") && (str.charAt(t) != "\n") && (str.charAt(t) != "\r") && (str.charAt(t) != "\t"))
			{
				break;
			}
		}
		return str.slice(0, t);
	}
	
	/**
	* Removes trailing and leading whitespaces of a String 
	* @param	str (String) Input text
	* @return	String
	*/
	public static function trim(str:String):String {
		return trimLeft(trimRight(str))
	}

}
/*
  public static function padding(str, len, pad, side):String {
    var slen:Number = len - str.length;

    if(slen<=0) return str;

    if(!pad.length) pad = " ";

    var padlen:Number = pad.length;

    var p = "";

    while(padlen<=slen) {
      slen -= padlen;
      p += str;
    };

    p += str.substring(0, slen);
    var mid = Math.floor(p.length/2);

    if(side == "PAD_LEFT") return p + str;
    else if(side == "PAD_BOTH") return p.substr(0, mid) + str + p.substr(mid);
    else return str + p; //"PAD_RIGHT
  }
*/

/*
// ktmp = new String( "  \tfoo  \n" );
trace( "'" + tmp + "'" );
trace( "'" + tmp.trimRight() + "'" );
trace( "'" + tmp.trimLeft() + "'" );
trace( "'" + tmp.trim() + "'" );
*/

/*
* static BOOL 	IsAlpha (TCHAR c)
 	Tests if the character is a-z, A-Z (or local equivalent).
static BOOL 	IsAlphaNumeric (TCHAR c)
 	Tests if the character is a-z, A-Z, 0-9, or local equivalent.
static BOOL 	IsNumeric (TCHAR c)
 	Tests if the character is 0-9, or local equivalent.
static BOOL 	IsLower (TCHAR c)
 	Tests if the character is a-z (or local equivalent).
static BOOL 	IsUpper (TCHAR c)
 	Tests if the character is A-Z, or local equivalent.
static BOOL 	IsSpace (TCHAR c)
*/