import com.fwa.system.FWAConfiguration;
import com.fwaapi.Log;
import com.fwaapi.Language;
import com.fwamarco.consoles.AbstractConsole;
import com.fwamarco.consoles.ConsolesManager;
import mx.utils.Delegate;

/**
* This class manages the Logs Console functionality.
* Integration with the ConsolesManager is managed by AbstractConsole.
* 
* @see		com.fwamarco.consoles.AbstractConsole
* @see 		com.fwamarco.consoles.ConsolesManager
* 
* @author	Federico Campo Piombi
* @version	1.3
* @since 	27 Nov 2006
* 
* @ignore
* MODIFICATION HISTORY
* 1.1 - 3 Jan 2007
* + Removed the Score.as integration
* + Removed the Score Display method
* 
* 1.2 - 6 Jan 2007
* + Added scroll capabilities
* 
* 1.3 - 10 Jan 2007
* + Added bt_clear
* + Replaced interval with "onUpdate" event from Log
* + Interval only updates the timer
* @ignore
*/
class com.fwamarco.consoles.Logs extends AbstractConsole {
	//layout elements
	private var clip_bg:MovieClip;
	private var bt_close:Button;
	private var bt_clear:Button;
	private var t_time:TextField;
	private var t_score:TextField;
	private var t_log:TextField;
	private var bt_scrollUp:Button;
	private var bt_scrollDown:Button;
	private var t_title_time:TextField;
	private var t_title_score:TextField;
	private var t_button_clear:TextField;
	
	//properties
	private var timer_int:Number;
	
	/**
	* Constructor
	*/
	public function Logs(){
		Log.onUpdate = Delegate.create (this, updateLogDisplay);
		bt_close.onRelease = Delegate.create(ConsolesManager, ConsolesManager.closeAll);
		bt_clear.onRelease = Delegate.create(Log, Log.clear);
		
		bt_scrollUp.onPress = Delegate.create(this, beginScrollUp);
		bt_scrollDown.onPress = Delegate.create(this, beginScrollDown);
		bt_scrollUp.onRelease = bt_scrollUp.onReleaseOutside = Delegate.create(this, endScroll);
		bt_scrollDown.onRelease = bt_scrollDown.onReleaseOutside = Delegate.create(this, endScroll);
		
		//bt_scrollUp.onRelease = Delegate.create(this, scrollUp);
		//bt_scrollDown.onRelease = Delegate.create(this, scrollDown);
		
		initTextFields();
	}
	
	/**
	* Displays the Log Console.
	* Starts the interval to update the Log's timer
	* @return 	Void
	*/
	public function show():Void {
		super.show();
		startTimeUpdater();
	}
	
	/**
	* Hides the Log Console
	* Stops the interval to update the Log's timer
	* @return	Void
	*/
	public function hide():Void {
		super.hide();
		stopTimeUpdater();
	}
	
	/**
	* Initialises the consoles TextFields content from the main language file
	* Updates the display properties of the related elements
	*/
	private function initTextFields():Void {
		//get TextFormats
		var tf_time:TextFormat = t_title_time.getTextFormat();
		var tf_score:TextFormat = t_title_score.getTextFormat();
		var tf_clear:TextFormat = t_button_clear.getTextFormat();
		
		//apply labels
		t_title_time.text = Language.getLabel("logs_tiempo");
		t_title_time.autoSize = true;
		t_title_score.text = Language.getLabel("logs_score");
		t_title_score.autoSize = true;
		t_button_clear.text = Language.getLabel("logs_vaciar");
		t_button_clear.autoSize = true;
		
		//apply TextFormats
		t_title_time.setNewTextFormat(tf_time);
		t_title_score.setNewTextFormat(tf_score);
		t_button_clear.setNewTextFormat(tf_clear);
		
		//update elements around
		t_time._x = t_title_time._x + t_title_time._width + 5
		t_score._x = t_title_score._x + t_title_score._width + 5
	}
	
	/**
	* Updates the Log History display
	* Formats the output Log content
	* @return	Void
	*/
	private function updateHistory():Void {
		var events:Array = Log.retrieveFull();
		var e_count:Number = events.length;
		t_log.text = ""
		t_log.htmlText = "<span>"
		for(var i:Number = 0; i<e_count; i++){
			var session:Object = events[i];
			var s_count:Number = session.history.length;
			
			t_log.htmlText += "<b>"+session.name+"</b><br>";
			
			for(var j:Number = 0; j<s_count; j++){
				var event:Object = session.history[j];
				t_log.htmlText += "<li><b>"+event.time+"</b> "+event.message+"</li>";
				t_log.htmlText += (j==s_count-1)? "<br>" : "";
			}
			
		}
		t_log.htmlText += "</span>"
	}
	
	/**
	* Updates the Time Display
	* Updates the Log History aswell
	* @return	Void
	*/
	private function updateLogDisplay():Void {
		updateHistory();
		updateScrolls();
		//trace("us:"+updateScrolls)
	}
	
	/**
	* Updates the console's timer
	*/
	private function updateTimer():Void {
		t_time.text = Log.getTime();
	}
	
	/**
	* Starts the interval to update the Time display
	* @return	Void
	*/
	private function startTimeUpdater():Void {
		stopTimeUpdater();
		timer_int = setInterval(this, "updateTimer", 500);
	}
	
	/**
	* Stops the Time Display interval
	* @return	Void
	*/
	private function stopTimeUpdater():Void {
		clearInterval(timer_int);
	}
	
	/**
	* Starts a loop to scroll up
	*/
	private function beginScrollUp():Void {
		this.onEnterFrame = Delegate.create(this, scrollUp);
	}
	
	/**
	* Starts a loop to scroll down
	*/
	private function beginScrollDown():Void {
		this.onEnterFrame = Delegate.create(this, scrollDown);
	}
	
	/**
	* Finishes scrolling the TextField
	*/
	private function endScroll():Void {
		delete this.onEnterFrame;
	}
	
	/**
	* Scrolls the TextField up
	*/
	private function scrollUp():Void {
		t_log.scroll ++
	}
	
	/**
	* Scrolls the TextField down
	*/
	private function scrollDown():Void {
		t_log.scroll --
	}
	
	/**
	* Updates the scroll buttons visibility according to the
	* Log's TextField scroll properties
	*/
	private function updateScrolls():Void {
		//trace("max:"+t_log.maxscroll+" bott:"+t_log.bottomScroll+" scroll:"+t_log.scroll);
		var hasScroll:Boolean = (t_log.maxscroll >1);
		bt_scrollUp._visible = hasScroll;
		bt_scrollDown._visible = hasScroll;
	}
}