﻿import com.fwamarco.consoles.ConsolesManager;
import com.fwaapi.Subtitles;
import com.fwaapi.Application;
import com.fwaapi.Language;
import com.fwaapi.Tooltip;
import com.fwaapi.Barrido;
import com.fwa.system.FWASystem;
import com.fwa.system.FWAConfiguration;
import com.fwa.debug.Tracer;
import mx.utils.Delegate;

/**
* This class initialises all the Marco Assets
* All the assets are marked as Unaccesible to avoid Screen Readers loop through main 
* controls.
* 
* @version	1.5
* @since 	05 Nov 2006
*/
class com.fwamarco.ui.Marco extends MovieClip {
	
	private var config:Object; //stores the mainconfig.xml data
	
	//layout elements
	private var txt_titulo:TextField;
	private var txt_subtitulos:TextField;
	private var bt_close:Button;
	private var clip_appselector:MovieClip;
	private var main_background:MovieClip;
	private var dither_bg:MovieClip;
	private var clip_titulo:MovieClip;
	private var clip_logo:MovieClip;
	private var clip_controles:MovieClip;
	private var clip_botones:MovieClip;
	private var clip_cerrar:MovieClip;
	private var clip_disablearea:MovieClip;
	
	private var alphaTo:Function; //inherited from the MovieClipTween prototypes. Declared here to avoid compile-time errors
	
	
	/**
	* Constructor: Initialises the Marco Configuration load.
	*/
	public function Marco(){
		this._visible = false
		//this._alpha = 0
		config = FWAConfiguration.main;
	}
	
	
	/**
	* Initialises the main components of the Marco and stores the Main Configuration
	* in the com.fwa.system.Configuration under the identifier "mainconfig"
	* @return	Void
	*/
	private function init():Void {
		Subtitles.setDisplay(txt_subtitulos);
		Barrido.setBlocker(clip_disablearea);
		
		txt_titulo.text = Application.getApplicationName();
		addEvents();
		addTooltips();
		applyColors();
		loadLogo();
		
		this._visible = true
		//this.alphaTo(100, 1, "easeOutQuint")
		
		//determine the amount of applications to display in the list
		//if none, disable the appselection button
		var apps:Number = FWAConfiguration.main.applications.app.length;
		var mode:String = (apps>1)? "on" : "off";
		trace("APPSELECTOR MODE: "+mode +" apps:"+String(apps) );
		clip_appselector.gotoAndStop(mode);
	}
	
	/**
	* Registers all the buttons events
	* @return	Void
	*/
	private function addEvents():Void {
		bt_close.onRelease = Delegate.create(this, closeApplication);
		clip_appselector.onSelect = Delegate.create(this, openApplicationsConsole);
	}
	
	/**
	* Applies the defined colors to all elements
	* @return	Void
	*/
	private function applyColors():Void {
		var color_config:Object = config.marco.colors
		main_background.colorTo("0x"+ color_config.background._value,0)
		dither_bg.clip_bg.colorTo("0x"+ color_config.marco._value,0)
		clip_titulo.clip_bg.colorTo("0x"+ color_config.title._value,0)
		clip_controles.clip_bg.colorTo("0x"+ color_config.controls._value,0)
		clip_cerrar.clip_bg.colorTo("0x"+ color_config.button_close._value,0)
	}
	
	/**
	* Loads the configured logo
	* @return	Void
	*/
	private function loadLogo():Void {
		var ref = this
		var file:String = config.marco.logo.file._value
		var mcl:MovieClipLoader = new MovieClipLoader();
		var lst:Object = new Object();
		lst.onLoadInit = function(tmc:MovieClip){
			tmc.onRelease = function(){
				getURL(ref.config.marco.logo.url._value, "_blank")
			}
			tmc.onRollOver = function(){
				Tooltip.show(Language.getLabel("tooltip_logo"))
			}
			tmc.onRollOut = tmc.onReleaseOutside = function(){
				Tooltip.hide()
			}
		}
		mcl.addListener(lst)
		mcl.loadClip(file, clip_logo.content)
	}
	
	/**
	* Sets Marco's elements tooltips
	* @return	Void
	*/
	private function addTooltips():Void {
		//clip_titulo
		clip_titulo.useHandCursor = false
		clip_titulo.onRollOver = function(){
			Tooltip.show(Language.getLabel("tooltip_titulo"))
		}
		clip_titulo.onRollOut = clip_titulo.onReleaseOutside = function(){
			Tooltip.hide();
		}
		
		//bt_close
		bt_close.onRollOver = function(){
			Tooltip.show(Language.getLabel("tooltip_cerrar"))
		}
		bt_close.onRollOut = clip_titulo.onReleaseOutside = function(){
			Tooltip.hide();
		}
		
		//selector apps
		clip_appselector.onOver = function(){
			Tooltip.show(Language.getLabel("tooltip_seleccion_aplicacion"));
		}
		clip_appselector.onOut = function(){
			Tooltip.hide();
		}
	}
	
	
	
	/**
	* Opens the Applications Selector Console
	*/
	private function openApplicationsConsole():Void {
		ConsolesManager.displayConsole("consola_aplicaciones");
	}
	
	
	/**
	* Evaluates the mode the application is running (standalone player or plugin) and calls
	* the respective close method
	* @return	Void
	*/
	private function closeApplication():Void {
		var mode:String = System.capabilities.playerType
		switch(mode){
			case "StandAlone":
				fscommand("quit")
				break;
			case "PlugIn":
			case "ActiveX":
				getURL("javascript:cerrar();")
				break;
			default:
				Tracer.output("Closing Application..", "closeApplication", "Marco")
				break;
		}
	}
}
