import com.wunderman.events.ListenerBroadcaster
import com.wunderman.events.IListenerBroadcaster
import mx.utils.Delegate

/**
* @class	User (Static)
* @tooltip	This class provides methods to login and logout users, and retrieve user's info
* @author	Federico Campo Piombi
* @version	1.1
* @date		01 Nov 2006
* @modified 07 Nov 2006
*/
class com.parker.users.User {
	
	private static var broadcaster:ListenerBroadcaster = null;
	private static var usr_info:Object = null;
	private static var usr_chances:Number = 0;
	
	/**
	* @constructor
	*/
	private function User(){}
	
	
	//PUBLIC METHODS
	/**
	* @method	login
	* @tooltip	Provides access to login a user
	* @param	udata (Object) User's data object
	* @return	Void
	*/
	public static function login(udata:Object):Void {
		usr_info = udata
		var chances:Number = (usr_info.user_info.chances._value!==undefined)? Number(usr_info.user_info.chances._value) : 0
		updateChances(chances)
		onLogin()
	}
	
	/**
	* @method	logout
	* @tooltip	Provides access to logout a user
	* @return	Void
	*/
	public static function logout():Void {
		usr_info = null
		onLogout()
	}
	
	/**
	* @method	isLogged
	* @tooltip	Returns true if the user_info Object is not null. 
	* 			If that object has data means that a user is logged.
	* @return	Boolean
	*/
	public static function isLogged():Boolean {
		return( (usr_info == null)? false : true )
	}
	
	/**
	* @method 	getInfo
	* @tooltip 	Returns the stored User's info
	* @return 	Object
	*/
	public static function getInfo():Object {
		return(usr_info.user_info) 
	}
	
	/**
	* @method 	getChances
	* @tooltip 	Returns the accumulated chances for the user
	* @return	Number
	*/
	public static function getChances():Number {
		return (usr_chances)
	}
	
	/**
	* @method 	updateChances
	* @tooltip 	Updates the chances accumulated by the user
	* @param	val (Number) The new amount of chances
	* @return	Void
	*/
	public static function updateChances(val:Number):Void {
		usr_chances = val
	}
	
	
	//PRIVATE METHODS
	/**
	* @method	onLogin
	* @tooltip	Executed when a user is succesfully logged in.
	* 			Broadcast a "onLogin" message to all subscriptors
	* @return	Void
	*/
	private static function onLogin():Void {
		broadcaster.broadcastMessage("onLogin")
	}
	
	/**
	* @method	onLogout
	* @tooltip	Executed when a user is succesfully logged out.
	* 			Broadcast a "onLogout" message to all subscriptors
	* @return	Void
	*/
	private static function onLogout():Void {
		broadcaster.broadcastMessage("onLogout")
	}
	
	
	//LISTENER BROADCASTER INTEGRATION
	public static function addListener(obj:Object):Boolean {
		if(broadcaster == null){ broadcaster = new ListenerBroadcaster() }
		return( broadcaster.addListener(obj) )
	}
	
	public static function removeListener(obj:Object):Boolean {
		if(broadcaster == null){ broadcaster = new ListenerBroadcaster() }
		return( broadcaster.removeListener(obj) )
	}
}