﻿import com.wunderman.core.ListenerBroadcaster

class com.wunderman.utils.ProgressiveLoader extends com.wunderman.core.Collection{
	
	var idle:Boolean
	var paused:Boolean
	
	var clip_dummy:MovieClip
	var clip_loader:MovieClipLoader
	var clip_listener:Object
	
	private var Broadcaster:ListenerBroadcaster
	
	public function ProgressiveLoader(dummy:MovieClip){
		super()
		this.clip_dummy = (dummy==undefined)? this.createDummyClip() : dummy
		this.idle = true
		this.paused = false
		this.clip_loader = new MovieClipLoader
		this.createLoadListener() //listener que retransmite los eventos onLoad, onInit, onProgress de los MCs
		this.Broadcaster = new ListenerBroadcaster() //broadcaster interno de la clase
	}
	
	/*
	* Function:	addElement
	* 
	* 	Adds an element to the ProgressiveLoader's collection.
	* 
	* Params:
	* 	filename - String: File to load
	* 	
	* 	holder - MovieClip: Target clip to load filename
	* 
	* 	listener - Object: Object to notify with load events
	* 
	* 	passive_mode - Boolean: To add an element without inmediat load
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myProgressiveLoader.addElement("file.jpg", this.content_clip, this.myListenerObject);
	* (end)
	* 
	* See Also:
	* 
	* <removeElement>
	* 
	* <clear>
	*/
	public function addElement(filename:String, holder:MovieClip, listener:Object, passive_mode:Boolean):Boolean {
		//if no holder provided to perform load, use the Dummy
		holder = (holder==undefined)? this.clip_dummy : holder
		//create Object to pass to Collection
		var obj:Object = new Object({filename:filename, holder:holder, listener:listener})
		//send object to Collection
		var success:Boolean = ( super.addElement(obj) )
		//if was sent OK (and passive_mode is not TRUE), process Queue, otherwise return false
		if(success==true && passive_mode!==true){
			this.processQueue()
		}else{
			return (false)
		}
		return (true)
	}
	
	/*
	* Function:	removeElement
	* 
	* 	Removes an element from the ProgressiveLoader's queue.
	* 
	* Params:
	* 	obj - Object: Object to remove from Queue
	* 
	* Returns:
	* 	Boolean
	* 
	* See Also:
	* <addElement>
	* 
	* <clear>
	*/
	private function removeElement(filename:String, holder:MovieClip, listener:Object){
		//if no holder provided to perform load, then assume Dummy was used
		holder = (holder==undefined)? this.clip_dummy : holder
		//create Object to pass to Collection
		var obj:Object = new Object({filename:filename, holder:holder, listener:listener})
		//tell Collection to remove this object
		return ( super.removeElement(obj) )
	}
	
	
	/*
	* Function:	clear
	* 
	* 	Empties de ProgressiveLoader's queue.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* Usage:
	* (begin code)
	* 	myProgressiveLoader.clear();
	* (end)
	* 
	* See Also:
	* <removeElement>
	* 
	* <pause>
	*/
	public function clear(){
		trace("PL: clear")
		this.pause()
		super.removeAll()
	}
	
	
	/*
	* Function:	pause
	* 
	* 	Pauses/Resumes the ProgressiveLoader queue.
	* 
	* Params:
	* 	resume - Boolean: If true, resumes the queue process (if previously paused)
	* 
	* Returns:
	* 	Nothing
	* 
	* Usage:
	* (begin code)
	* 	myProgressiveLoader.pause(); //pauses the queue
	* 
	* 	myProgressiveLoader.pause(true); //resumes the queue
	* (end)
	* 
	* See Also:
	* 
	* <clear>
	*/
	public function pause(resume:Boolean):Void {
		if(resume == true){
			if(this.paused == true){
				this.paused = false
				this.processQueue()
			}
		}else{
			if(this.paused==false){
				this.paused = true
				//this.pauseQueue()
			}
		}
	}
	
	
	
	/*
	* Function:	processQueue
	* 
	* 	Starts loading the next element of the queue.
	* 	Sets the ProgressiveLoader's *idle* status to *true*
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <pauseQueue>
	* 
	*/
	public function processQueue(){
		//if loader is Idle, and there are elements left in the Collection, process
		if(this.idle == true && this.paused == false && super.getElementsCount()>0){
			//if next element is a valid element (has holder, and has filename)
			var elem_filename:String = super.getElement().filename
			var elem_holder:MovieClip = super.getElement().holder
			if(elem_filename!==undefined && elem_filename!== "" && elem_holder!==undefined){
				//perform load of next Element
				this.attachListener()
				this.clip_loader.loadClip(elem_filename, elem_holder)
				this.idle = false
			}
		}else{
			
			//in case Queue can't be processed
			if(super.getElementsCount()==0){
				//queue empty
				this.Broadcaster.broadcastMessage("onEmptyQueue")
			}
		}
	}
	


	/*
	* Function:	pauseQueue
	* 
	* 	- Sends the *onLoadPaused* message to the currently loading clip.
	* 	- Unloads the currently loading filename
	* 	- Removes the progress listener
	* 	- Sets the *idle* status to *true*
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <processQueue>
	* 
	* <removeLastProcessedElement>
	* 
	*/
	private function pauseQueue(){
		this.broadcastMessage("onLoadPaused")
		this.clip_loader.unloadClip(super.getElement().holder)
		this.deattachListener()
		this.idle = true
	}
	

	/*
	* Function:	removeLastProcessedElement
	* 
	* 	Removes the currently processed Queue's element.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <pauseQueue>
	* 
	*/
	private function removeLastProcessedElement(){
		return (super.removeElement( super.getElement() ) )
	}
	

	/*
	* Function:	broadcastMessage
	* 
	* 	Sends a message to the current element's Listener.
	* 
	* Params:
	* 	msg - String: String with the message to execute
	* 
	* 	parameters - Array: Array of parameters to pass to the Listener
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <createLoadListener>
	* 
	* <pauseQueue>
	* 
	*/
	private function broadcastMessage(msg:String, parameters:Array){
		super.getElement().listener[msg].apply(null,parameters)
	}
	
	
	/*
	* Function:	createDummyClip
	* 
	* 	Creates a dummy clip to preload images, if *holder* was not set as parameter in *addElement*.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <pauseQueue>
	* 
	*/
	private function createDummyClip(){
		var cdummy:MovieClip = (!_root.__dummy)? _root.createEmptyMovieClip("__dummy",_root.getNextHighestDepth()) : _root.__dummy
		cdummy._visible = false
		return (cdummy)
	}
	


	/*
	* Function:	attachListener
	* 
	* 	Attachs the ProgressiveLoader's main listener to the current element's holder clip
	* 	This Listener re-broadcast to the Element's listener events, and also executes the
	* 	*processQueue* method when it finishes loading the image.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <createLoadListener>
	* 
	* <deattachListener>
	* 
	*/
	private function attachListener():Void {
		this.clip_loader.addListener(this.clip_listener)
	}
	
	
	/*
	* Function:	deattachListener
	* 
	* 	Deattachs the ProgressiveLoader's main listener from the current element's holder clip
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <attachListener>
	* 
	*/
	private function deattachListener():Void {
		this.clip_loader.removeListener(this.clip_listener)
	}
	

	/*
	* Function:	createLoadListener
	* 
	* 	Creates the ProgressiveLoader's main listener.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	Nothing
	* 
	* See Also:
	* <attachListener>
	* 
	* <deattachListener>
	* 
	*/
	private function createLoadListener():Void {
		var tRef = this
		this.clip_listener = new Object()
		this.clip_listener.onLoadStart = function(tmc:MovieClip){
			tRef.broadcastMessage("onLoadStart", [tmc])
		}
		this.clip_listener.onLoadProgress = function(tmc:MovieClip, bL:Number, bT:Number){
			tRef.broadcastMessage("onLoadProgress", [tmc,bL,bT])
		}
		this.clip_listener.onLoadInit = function(tmc:MovieClip){
			tRef.broadcastMessage("onLoadInit",[tmc])
			_global.tb.trace("\nLOAD SUCCESS:"+tmc)
			if (tRef.removeLastProcessedElement() == true){
				tRef.idle = true
				tRef.processQueue()
			}
		}
		this.clip_listener.onLoadError = function(tmc:MovieClip, errorString:String, httpStatus:Number){
			//in case of ANY error, broadcast and proceed with next queued file
			_global.tb.trace("\nLOAD ERROR:"+errorString)
			tRef.broadcastMessage("onLoadError",[tmc, errorString, httpStatus])
			if (tRef.removeLastProcessedElement() == true){
				tRef.idle = true
				tRef.processQueue()
			}
		}
	}
	
	//LISTENER BROADCASTER INTEGRATION///////////////////////////////////////////////////////////
	/*
	* Function:	addListener
	* 
	* 	- Adds an object to be notified of Collection's events.
	* 
	* Params:
	* 	objListener - Object: Object with the methods to be notified
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.addListener(myListenerObject);
	* (end)
	* 
	* See Also:
	* 	<removeListener>
	*/
	public function addListener(obj:Object){
		return (this.Broadcaster.addListener(obj))
	}
	
	
	/*
	* Function:	removeListener
	* 
	* 	- Removes an object from the event's notification list
	* 
	* Params:
	* 	objListener - Object: Object to be removed
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.removeListener(myListenerObject);
	* (end)
	* 
	* See Also:
	* 	<addListener>
	*/
	public function removeListener(obj:Object){
		return (this.Broadcaster.removeListener(obj))
	}

}