﻿import com.fwa.events.ListenerBroadcaster
/**
* Collection Class
* This class is deprecated.
* All references to this class should be made to com.asdk (Many different collection types)
* 
* @deprecated
*/
class com.fwa.core.Collection{
	var Broadcaster:ListenerBroadcaster
	var elements:Array
	
	public function Collection(){
		this.Broadcaster = new ListenerBroadcaster()
		this.elements = new Array()
	}
		
	/*
	* Function:	addElement
	* 	- Adds an element to the Collection's tail.
	* 	- When adding elements to the Collection, this method first checks that the element doesn't already exists in the Collection.
	* 	If it does, then element is not inserted EXCEPT "force_insert" is set to TRUE.
	* 
	* Params:
	* 	elem - Object: Object to add to the Collection
	*	force_insert -  Boolean: forces the object into the Collection even if it already exists
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.addElement(myObj);
	* 	
	* 	myCollection.addElement(myObj,true);
	* (end)
	* 
	* See Also:
	* <removeElement>
	*
	* <replaceElement>
	* 
	* <existsElement>
	* 
	*/
	public function addElement(elem:Object, force_insert:Boolean):Boolean {
		if(this.existsElement(elem)==false || force_insert==true){
			this.elements.push(elem)
			this.Broadcaster.broadcastMessage("onElementAdded",[elem])
			return (true)
		}else{
			return (false)
		}
	}
	
	
	/*
	* Function: replaceElement
	* 	- Replaces an exisiting element from the Collection with another one.
	* 
	* Params:
	* 	old_element - Object: Object to replace from the Collection
	*	new_element - Object: Obejct to replace with
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.replaceElement(myOldObj, myNewObj);
	* (end)
	* 
	* See Also:
	* <addElement>
	*
	* <removeElement>
	* 
	*/
	public function replaceElement(elem_old:Object, elem_new:Object):Boolean {
		if(this.existsElement(elem_old)){
			var element_position:Number = this.getIDByElement(elem_old)
			this.elements[element_position] = elem_new
			this.Broadcaster.broadcastMessage("onElementUpdated", [elem_old, elem_new, element_position])
			return (true)
		}else{
			return (false)
		}
	}
	
	/*
	* Function:	removeElement
	* 
	* 	- Removes an exisiting element from the Collection.
	* 
	* Params:
	* 	elem - Object: Object to remove from the Collection
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.removeElement(myObj);
	* (end)
	* 
	* See Also:
	* <removeAll>
	* 
	* <addElement>
	*
	* <replaceElement>
	* 
	*/
	public function removeElement(elem:Object):Boolean{
		if(this.existsElement(elem)){
			var elem_id:Number = this.getIDByElement(elem)
			var removed:Object = this.elements.splice(elem_id,1)
			this.Broadcaster.broadcastMessage("onElementRemoved",[removed,elem_id])
			return(true)
		}else{
			return (false)
		}
	}
	
	
	/*
	* Function:	removeAll
	* 
	* 	- Empties the Collection.
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	- Nothing
	* 
	* Usage:
	* (begin code)
	* 	myCollection.removeAll();
	* (end)
	* 
	* See Also:
	* <removeElement>
	* 
	* <addElement>
	*
	* <replaceElement>
	* 
	*/
	public function removeAll():Void{
		this.elements = new Array()
		this.Broadcaster.broadcastMessage("onElementsRemoved")
	}
	
	
	/*
	* Function:	existsElement
	* 
	* 	- Checks if the given Object exists in the elements Collection.
	* 
	* Params:
	* 	elemen - Object: Object to search for in the Collection
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.existsElement(myObj);
	* (end)
	* 
	* See Also:
	* <getElementByID>
	*
	* <getIDByElement>
	* 
	*/
	public function existsElement(elem:Object):Boolean {
		return ( (this.getIDByElement(elem)==-1)? false : true )
	}
	
	
	/*
	* Function:	getElement
	* 
	* 	- Returns the first element from the Collection (Collection[0])
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	- Object
	* 
	* Usage:
	* (begin code)
	* 	myCollection.getElement()
	* (end)
	* 
	* See Also:
	* 	<getElementByID>
	* 
	* 	<getIDByElement>
	* 
	* 	<existsElement>
	* 
	* 	<getElements>
	*/
	public function getElement():Object{
		return(this.elements[0])
	}
	
	
	/*
	* Function:	getElements
	* 
	* 	- Returns all elements from the Collection
	* 
	* Params:
	* 	none - 
	* 
	* Returns:
	* 	- Array
	* 
	* Usage:
	* (begin code)
	* 	myCollection.getElements()
	* (end)
	* 
	* See Also:
	* 	<getElementByID>
	* 
	* 	<getIDByElement>
	* 
	* 	<existsElement>
	* 
	* 	<getElement>
	*/
	public function getElements():Array{
		return(this.elements)
	}
	
	
	/*
	* Function:	getElementByID
	* 
	* 	- Given an ID, returns the according element from the Collection (Collection[ID])
	* 
	* Params:
	* 	id - Number
	* 
	* Returns:
	* 	- Object
	* 
	* Usage:
	* (begin code)
	* 	myCollection.getElementByID(5)
	* (end)
	* 
	* See Also:
	* 	<getElement>
	* 
	* 	<getElements>
	* 
	* 	<getIDByElement>
	* 
	* 	<existsElement>
	*/
	public function getElementByID(id:Number):Object {
		return(this.elements[id])
	}


	/*
	* Function:	getIDByElement
	* 
	* 	- Given an Elemen, this method locates it in the Collection and returns it ID (or -1 if not found)
	* 
	* Params:
	* 	elem - Object
	* 
	* Returns:
	* 	- Number
	* 
	* Usage:
	* (begin code)
	* 	myCollection.getElementByID(5)
	* (end)
	* 
	* See Also:
	* 	<getElement>
	* 
	* 	<getElements>
	* 
	* 	<getElementByID>
	* 
	* 	<existsElement>
	*/
	public function getIDByElement(elem:Object):Number {
		for(var i=0; i<this.getElementsCount(); i++){
			if(elem == this.elements[i]){
				return(i)
			}
		}
		return (-1)
	}
	
	
	/*
	* Function:	findElement
	* 
	* 	- retunrs the first element found that has the given variable with the given value
	* 
	* Params:
	* 	elem_var - Object
	*
	*   elem_var_value - Object
	* 
	* Returns:
	* 	- Element
	* 
	* Usage:
	* (begin code)
	* 	myCollection.findElement("variable", "value")
	* (end)
	* 
	* See Also:
	* 	<getElement>
	* 
	* 	<getElements>
	* 
	* 	<getElementByID>
	* 
	* 	<getIDByElement>
	* 
	* 	<existsElement>
	*/
	public function findElement(elem_var:Object, elem_var_value:Object):Object {
		for(var i=0; i<this.getElementsCount(); i++){
			var tElem = this.elements[i]
			if(tElem[elem_var]!==undefined && tElem[elem_var] == elem_var_value){
				trace("COLLECTION: Element Found")
				return(tElem)
			}
		}
		trace("COLLECTION: elem not found")
		return (undefined)
	}
	
	
	/*
	* Function:	getElementsCount
	* 
	* 	- Returns the Collection's elements count.
	* 
	* Params:
	* 	none
	* 
	* Returns:
	* 	Number
	* 
	* Usage:
	* (begin code)
	* 	myCollection.getElementsCount();
	* (end)
	*/
	public function getElementsCount():Number{
		return (this.elements.length)
	}
	
	
	//LISTENER BROADCASTER INTEGRATION///////////////////////////////////////////////////////////
	/*
	* Function:	addListener
	* 
	* 	- Adds an object to be notified of Collection's events.
	* 
	* Params:
	* 	objListener - Object: Object with the methods to be notified
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.addListener(myListenerObject);
	* (end)
	* 
	* See Also:
	* 	<removeListener>
	*/
	public function addListener(obj:Object){
		return (this.Broadcaster.addListener(obj))
	}
	
	
	/*
	* Function:	removeListener
	* 
	* 	- Removes an object from the event's notification list
	* 
	* Params:
	* 	objListener - Object: Object to be removed
	* 
	* Returns:
	* 	Boolean
	* 
	* Usage:
	* (begin code)
	* 	myCollection.removeListener(myListenerObject);
	* (end)
	* 
	* See Also:
	* 	<addListener>
	*/
	public function removeListener(obj:Object){
		return (this.Broadcaster.removeListener(obj))
	}
}
