import com.fwa.system.FWAConfiguration;
import com.fwa.data.NodeRetriever;

/**
* Static class Subtitles
* This class works as an API to display subtitles in the Framework, by implementing the merge between the 
* FWA Configuration and the processes to search throug that config's content for the given subtitle.
*/
class com.fwaapi.Subtitles {
	
	private static var subs_output:TextField;
	
	
	/**
	* Constructor 
	*/
	private function Subtitles(){}
	
	
	/**
	* Displays a subtitle in the default display.
	* @param 	identifier (String) Identifier of the node to display
	* @return	Void
	*/
	public static function display(identifier:String):Void {
		displayAt(identifier, subs_output)
	}
		
	/**
	* Displays a subtitle in the given output textfield
	* @param	identifier (String) Identifier of the node to display
	* @param	output (TextField) Textfield where the node's content will be displayed
	* @return	Void
	*/
	public static function displayAt(identifier:String, output:TextField):Void {
		output.text = getSubtitle(identifier)
	}
	
	/**
	* Clears the output textfield content
	* @return	Void
	*/
	public static function clear():Void {
		subs_output.text = ""
	}
	
	/**
	* Sets the default subtitles display TextField. 
	* This TextField will be used to display subtitles when ivoking <display>
	* @param	output (TextField) TextField to display the subtitles
	*/
	public static function setDisplay(output:TextField):Void {
		subs_output = output
	}
	
	/**
	* Retrieves a reference to the default subtitles display.
	* @return 	TextField
	*/
	public static function getDisplay():TextField {
		return subs_output
	}
	
	/**
	* This method returns a String with the content of the given subtitle's identifier
	* @param	identifier (String) Identifier of the subtitle
	* @return	String
	*/
	public static function getSubtitle(identifier:String):String {
		var content:String = getNodeContent(identifier)
		return(content)
	}
	
	
	//PRIVATE METHODS
	/**
	* Retrieves the given node's identifier content text.
	* @param	identifier (String) Identifier of the node whose content needs to be retrieved
	* @return	String
	*/
	private static function getNodeContent(identifier:String):String {
		var obj:Object = FWAConfiguration.subtitles
		var node:Object = NodeRetriever.retrieveFirstNode(identifier, obj.subtitle)
		var node_content:String = node.content._value
		
		return( (node_content==undefined)? "" : node_content )
	}
}