import mx.utils.Delegate;

/**
* This class creates an Abstract Button to be used in the Applications.
* This class defines default onPress, onRelease, onRollOver, onRollOut and onReleaseOutside methods 
* that can't be overwritten. Although you can add new events to all AbstractButtons, 
* the original events defined here are executed before the new ones you define.
* This way, instead of a Button you can have a MovieClip with 3 frames (button status) and this class will
* perform the default rollOver, rollOut and press actions of a button, allowing you to code the actions only.
* 
* @version	1.0
* @since 	11 Dec 2006
*/
class com.fwaapi.buttons.AbstractButton extends MovieClip {
	
	//properties
	private var custom_rollOver:Array;
	private var custom_rollOut:Array;
	private var custom_releaseOutside:Array;
	private var custom_press:Array;
	private var custom_release:Array;
	
	
	/**
	* Constructor
	*/
	public function AbstractButton(){
		stop()
		custom_rollOver = new Array();
		custom_rollOut = new Array();
		custom_releaseOutside = new Array();
		custom_press = new Array();
		custom_release = new Array();
		
		//watch events
		this.watch("onPress", watchEvent, custom_press)
		this.watch("onRelease", watchEvent, custom_release)
		this.watch("onRollOver", watchEvent, custom_rollOver)
		this.watch("onRollOut", watchEvent, custom_rollOut)
		this.watch("onReleaseOutside", watchEvent, custom_releaseOutside)
	}
	
	
	/**
	* Default onPress Action
	*/
	public function onPress():Void {
		gotoAndStop(3);
		executeCustomActions(custom_press);
	}
	
	/**
	* Default onRelease Action
	*/
	public function onRelease():Void {
		gotoAndStop(1);
		executeCustomActions(custom_release);
	}
	
	/**
	* Default onRollOver Action
	*/
	public function onRollOver():Void {
		gotoAndStop(2);
		executeCustomActions(custom_rollOver);
	}
	
	/**
	* Default onRollOut Action
	*/
	public function onRollOut():Void {
		gotoAndStop(1);
		executeCustomActions(custom_rollOut);
	}
	
	/**
	* Default onReleaseOutside Action
	*/
	public function onReleaseOutside():Void {
		gotoAndStop(1);
		executeCustomActions(custom_releaseOutside);
	}
	
	/**
	* Sets a watchpoint to avoid overriding the default button events. All new events defined are passed through
	* this watchpoint and added to the custom_event array, so the original event defined is executed.
	* @param	event (String) Event being modified
	* @param	oldVal (Object) Old value for the event
	* @param	newVal (Object) Value trying to be assigned to that event
	* @param	actions_array (Array) Array where the new Value should be added to
	* @return	Object
	*/
	private function watchEvent(event:String, oldVal:Object, newVal:Object, actions_array:Array):Object {
		actions_array.push(newVal)
		return(oldVal)
	}
	
	/**
	* Executes the actions in the given actions array
	* @param	actions (Array) Array with the actions to execute
	* @return	Void
	*/
	private function executeCustomActions(actions:Array):Void {
		var customs:Number = actions.length
		for(var i:Number = 0; i<customs; i++){
			var action:Function = actions[i]
			action.apply(null, [])
		}
	}
}