import mx.utils.Delegate
import com.wunderman.utils.XMLConstruct
import com.wunderman.debug.Tracer

/**
* @class	Configuration (Singleton)
* @author	Federico Campo Piombi
* @tooltip	This class provides access to a main configuration file content.
* @version	1.0
* @date		01 Nov 2006
*/
class com.parker.core.Configuration {
	private static var elements_loaded:Number = 0 //counter to check how many elements were loaded
	private static var elements_total:Number = 5 //total of elements that must be loaded to fire the onLoad event
	private static var config:Object
	private static var structure:Object
	private static var paises:Object
	private static var destinos:Object
	private static var modelos:Object
	
	private static var _instance:Configuration = null
	
	
	/**
	* @constructor
	*/
	private function Configuration(){}
	
	//PUBLIC METHODS
	/**
	* @method	getInstance
	* @tooltip	Retunrs a single instance of the current class.
	* @return	Configuration
	*/
	public static function getInstance():Configuration {
		if(_instance == null){
			return new Configuration()
		}else{
			return _instance
		}
	}
	
	/**
	* @method	init
	* @tooltip	Initialises the <config> Object by loading the external configuration file
	* @return	Void
	*/
	public static function init():Void {
		if(elements_loaded == elements_total){
			onLoad()
		}else{
			config = new XMLConstruct()
			config.onLoad = onElementLoaded
			config.load("config/config.xml")
			
			structure = new XMLConstruct()
			structure.onLoad = onElementLoaded
			structure.load("config/structure.xml")
			
			paises = new XMLConstruct()
			paises.onLoad = onElementLoaded
			paises.load("config/paises.xml")
			
			destinos = new XMLConstruct()
			destinos.onLoad = onElementLoaded
			destinos.load("config/destinos.xml")
			
			modelos = new XMLConstruct()
			modelos.onLoad = onElementLoaded
			modelos.load("config/modelos.xml")
		}
	}
	
	/**
	* @method	getConfig
	* @tooltip	Returns the stored configuration object
	* @return	Object
	*/
	public static function getConfig():Object {
		return config
	}
	
	/**
	* @method	getStructure
	* @tooltip	Returns the stored Structure object
	* @return	Object
	*/
	public static function getStructure():Object {
		return structure
	}
	
	/**
	* @method	getPaises
	* @tooltip	Returns the stored Paises object
	* @return	Object
	*/
	public static function getPaises():Object {
		return paises
	}
	
	/**
	* @method	getDestinos
	* @tooltip	Returns the stored Destinos object
	* @return	Object
	*/
	public static function getDestinos():Object {
		return destinos
	}
	
	/**
	* @method	getModelos
	* @tooltip	Returns the stored Modelos object
	* @return	Object
	*/
	public static function getModelos():Object {
		return modelos
	}
	
	
	//EVENTS
	/**
	* @method 	onElementLoaded
	* @tooltip 	Event executed when an element (file) is loaded
	* @return 	Void
	*/
	private static function onElementLoaded():Void {
		elements_loaded ++
		if(elements_loaded==elements_total){
			onLoad()
		}
	}
	
	/**
	* @method	onLoad
	* @tooltip	Event executed when all the XML files are loaded
	* @return	Void
	*/
	public static function onLoad():Void {}
}